/*******************************************************************************
* 
* The content of this file or document is CONFIDENTIAL and PROPRIETARY
* to Mobilygen Corporation.  It is subject to the terms of a
* License Agreement between Licensee and Mobilygen Corporation.
* restricting among other things, the use, reproduction, distribution
* and transfer.  Each of the embodiments, including this information and
* any derivative work shall retain this copyright notice.
* 
* Copyright 2004 Mobilygen Corporation
* All rights reserved.
* 
* QuArc is a registered trademark of Mobilygen Corporation.
* 
*******************************************************************************/

#ifndef __QHAL_HOST_H
#define __QHAL_HOST_H

#include <qhal_errno.h>

#define NINTERFACES   7
typedef enum {
    QHALHOST_BUS_CSR=0,
    QHALHOST_BUS_EM1=1,
    QHALHOST_BUS_EM1_STREAM=2,
    QHALHOST_BUS_EM2=3,
    QHALHOST_BUS_EM2_STREAM=4,
    QHALHOST_BUS_BITSTREAM=5,
    QHALHOST_INT_MBOX0=6,
    QHALHOST_INT_MBOX1=7,
    QHALHOST_INT_VIDEO=8
} QHALHOST_INTERFACE;

typedef enum {
    QHALHOST_BUSMODE_DISABLED,  /* This interface can't be used to transfer data.
                                    thus read/write operation can't be used
                                */
    QHALHOST_BUSMODE_WORD,      /* This interface can only transfer single data
                                    thus block read/write operation can't be used
                                */
    QHALHOST_BUSMODE_PIO_POLL,  /* poll ready signal and send threshold data, 
                                    chip layer (qhal except qhalhost) is 
                                    responsible for threshold control
                                    and polling
                                */
    QHALHOST_BUSMODE_DMA_WAIT,  /* use wait signal to pause transfer, host is responsible
                                    for setting up properly DMA engine and make sure
                                    number of data sent doesn't exceed host DMA engine capabilities.
                                    chip layer is responsible for making sure chip's DMA engine
                                    capabilities are not exceeded
                                */
    QHALHOST_BUSMODE_DMA_REQ    /* use req signal to continue transfer, host is responsible
                                    for setting up properly DMA engine and make sure
                                    number of data sent doesn't exceed host DMA engine capabilities.
                                    chip layer is responsible for making sure chip's DMA engine
                                    capabilities are not exceeded.
                                */
} QHALHOST_BUSMODE;

#define QHALHOST_GETSWAPMASK                 (0)
#define QHALHOST_DISABLESWAPMODE(swapmask)   (swapmask)
#define QHALHOST_BYTESWAP                    (1<<0)  /* Swap 8bits words */
#define QHALHOST_SHORTSWAP                   (1<<1)  /* Swap 16bits words */
#define QHALHOST_LONGSWAP                    (1<<2)  /* Swap 32bits words */

typedef int qhalhost_handle_t;

typedef int (*qhalhost_callback_t)(void *);

#ifdef    __cplusplus
extern "C" {
#endif /* __cplusplus */

/**qhalhost_open
 * Perform init for hostbus
 * \param   codec       The codec you want to access.
 * \param   interface   Which host interface the user wants to open
 * \return  a negative value means an error, and a positive value is a handler 
 */
qhalhost_handle_t qhalhost_open(int codec, QHALHOST_INTERFACE interface);

/**qhalhost_getbusmode
 * Return the mode of operations for the selected interface (PIO, DMA Wait, DMA Req)
 * \param   host_h  Handle returned by qhalqhalhost_open
 * \return  mode being used.
 */
QHALHOST_BUSMODE qhalhost_getbusmode(qhalhost_handle_t host_h);

/**qhalhost_swapmode
 * Get and Select byteswap modes for qhalhost_*_block APIs
 * \param   host_h      Handle returned by qhalqhalhost_open
 * \param   swapmode    If GETSWAPMASK, api will return current swapmode
 *                      otherwise will disable byteswap according to mask
 * \return  a negative value means an error, QHAL_SUCCESS otherwise
 */
int qhalhost_swapmode(qhalhost_handle_t host_h, int swapmode);

/**qhalhost_read_long
 * Read a 32bits word on host bus
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to read (relative to HOST_INTERFACE)
 * \return  data read
 */
unsigned long qhalhost_read_long(qhalhost_handle_t host_h, unsigned long addr);

/**qhalhost_read_short
 * Read a 16bits word on host bus
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to read (relative to HOST_INTERFACE)
 * \return  data read
 */
unsigned short qhalhost_read_short(qhalhost_handle_t host_h, unsigned long addr);

/**qhalhost_read_byte
 * Read a byte on host bus
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to read (relative to HOST_INTERFACE)
 * \return  data read
 */
unsigned char qhalhost_read_byte(qhalhost_handle_t host_h, unsigned long addr);

/**qhalhost_read_byte_block
 * Read a block of bytes on host bus at A FIXED ADDRESS
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to read (relative to HOST_INTERFACE)
 * \param   buffer  buffer where data will be stored
 * \param   nBytes  Number of bytes to read
 * \return  Number of bytes read, SHOULD BE nBytes if everything is ok
 *          A negative number means an error occured.
 * \note: This API will perform necessary byteswap depending on:
 *      1- host and device endianess (only necessary if devices have different endianess)
 *      2- data size
 *      3- bus size
 *      if the bus is 8bits -> no byteswap.
 *      if the bus is 16bits -> swap bytes inside each 16bits word.
 *      if the bus is 32bits -> swap byte 0 with byte 3 and byte 1 with byte 2 in each 32bits word.
 */
int qhalhost_read_byte_block(qhalhost_handle_t host_h, unsigned long addr, char *buffer, int nBytes);

/**qhalhost_read_short_block
 * Read a block of shorts (16bits words) on host bus at A FIXED ADDRESS (16bits aligned ...)
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to read (relative to HOST_INTERFACE)
 * \param   buffer  buffer where data will be stored
 * \param   nShorts Number of shorts to read
 * \return  Number of shorts read, SHOULD BE nShorts if everything is ok
 *          A negative number means an error occured.
 * \note: This API will perform necessary byteswap depending on:
 *      1- host and device endianess (only necessary if devices have different endianess)
 *      2- data size
 *      3- bus size
 *      if the bus is 8bits -> byteswap bytes in each 16bits word.
 *      if the bus is 16bits -> no byteswap.
 *      if the bus is 32bits -> we need to swap both 16bits words into the 32bits word.
 */
int qhalhost_read_short_block(qhalhost_handle_t host_h, unsigned long addr, short *buffer, int nShorts);

/**qhalhost_read_long_block
 * Read a block of longs (32bits words) on host bus at A FIXED ADDRESS (32bits aligned ...)
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to read (relative to HOST_INTERFACE)
 * \param   buffer  buffer where data will be stored
 * \param   nLongs  Number of longs to read
 * \return  Number of longs read, SHOULD BE nLongs if everything is ok
 *          A negative number means an error occured.
 * \note: This API will perform necessary byteswap depending on:
 *      1- host and device endianess (only necessary if devices have different endianess)
 *      2- data size
 *      3- bus size
 *      if the bus is 8bits -> swap byte 0 with byte 3 and byte 1 with byte 2 in each 32bits word.
 *      if the bus is 16bits -> swap bytes inside each 16bits word.
 *      if the bus is 32bits -> no swap.
 */
int qhalhost_read_long_block(qhalhost_handle_t host_h, unsigned long addr, long *buffer, int nLongs);

/**qhalhost_write_long
 * Write a 32bit word on host bus
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to write to (relative to HOST_INTERFACE)
 * \param   data    Data to write
 */
void qhalhost_write_long(qhalhost_handle_t host_h, unsigned long addr, unsigned long data);

/**qhalhost_write_short
 * Write a 16bit word on host bus
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to write to (relative to HOST_INTERFACE)
 * \param   data    Data to write
 */
void qhalhost_write_short(qhalhost_handle_t host_h, unsigned long addr, unsigned short data);

/**qhalhost_write_byte
 * Write a byte on host bus
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to write to (relative to HOST_INTERFACE)
 * \param   data    Data to write
 */
void qhalhost_write_byte(qhalhost_handle_t host_h, unsigned long addr, unsigned char data);

/**qhalhost_write_byte_block
 * Write a block of bytes on host bus at A FIXED ADDRESS
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to write to (relative to HOST_INTERFACE)
 * \param   buffer  buffer containing data to be sent
 * \param   nBytes  Number of bytes to write
 * \return  Number of bytes written, SHOULD BE nBytes if everything is ok
 *          A negative number means an error occured.
 * \note: This API will perform necessary byteswap depending on:
 *      1- host and device endianess (only necessary if devices have different endianess)
 *      2- data size
 *      3- bus size
 *      if the bus is 8bits -> no byteswap.
 *      if the bus is 16bits -> swap bytes inside each 16bits word.
 *      if the bus is 32bits -> swap byte 0 with byte 3 and byte 1 with byte 2 in each 32bits word.
 */
int qhalhost_write_byte_block(qhalhost_handle_t host_h, unsigned long addr, char *buffer, int nBytes);

/**qhalhost_write_short_block
 * Write a block of shorts (16bits words) on host bus at A FIXED ADDRESS
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to write to (relative to HOST_INTERFACE)
 * \param   buffer  buffer containing data to be sent
 * \param   nShorts Number of shorts to write
 * \return  Number of shorts written, SHOULD BE nShorts if everything is ok
 *          A negative number means an error occured.
 * \note: This API will perform necessary byteswap depending on:
 *      1- host and device endianess (only necessary if devices have different endianess)
 *      2- data size
 *      3- bus size
 *      if the bus is 8bits -> byteswap bytes in each 16bits word.
 *      if the bus is 16bits -> no byteswap.
 *      if the bus is 32bits -> we need to swap both 16bits words into the 32bits word.
 */
int qhalhost_write_short_block(qhalhost_handle_t host_h, unsigned long addr, short *buffer, int nShorts);

/**qhalhost_write_long_block
 * Write a block of longs (32bits words) on host bus at A FIXED ADDRESS
 * \param   host_h  Handle returned by qhalhost_open
 * \param   addr    Address on host bus to write to (relative to HOST_INTERFACE)
 * \param   buffer  buffer containing data to be sent
 * \param   nLongs  Number of longs to write
 * \return  Number of longs written, SHOULD BE nLongs if everything is ok
 *          A negative number means an error occured.
 * \note: This API will perform necessary byteswap depending on:
 *      1- host and device endianess (only necessary if devices have different endianess)
 *      2- data size
 *      3- bus size
 *      if the bus is 8bits -> swap byte 0 with byte 3 and byte 1 with byte 2 in each 32bits word.
 *      if the bus is 16bits -> swap bytes inside each 16bits word.
 *      if the bus is 32bits -> no swap.
 */
int qhalhost_write_long_block(qhalhost_handle_t host_h, unsigned long addr, long *buffer, int nLongs);

/**qhalhost_wait_interrupt
 * Wait for an interrupt to come
 * \param   host_h  Handle returned by qhalhost_open
 * \param   timeout Wait will fail if interrupt has not come after timeout microseconds
 * \return  QHAL_SUCCESS on success, see qhal_errno.h.
 */
int qhalhost_wait_interrupt(qhalhost_handle_t host_h, int timeout);

/**qhalhost_ack_interrupt
 * Acknowledge to the host that interrupt has been serviced.
 * This API is used by QHAL to notify host that this interface interrupt can be enabled again
 * (interrupt status has been cleared on QMM side)
 * \param   host_h  Handle returned by qhalhost_open
 * \return  QHAL_SUCCESS on success, see qhal_errno.h.
 */
int qhalhost_ack_interrupt(qhalhost_handle_t host_h);

/**qhalhost_wait
 * Wait for a specific time in ms
 * NOTE: This is an approximative wait and should be implemented as a context switch
 * on most host environment, thus allowing the host to do something else for a while
 * All that should be expected by API calling this method is that AT LEAST time_ms ms
 * will elapse.
 * NOTE2: If time_ms is 0 this API should return immediately if qhal is running in a
 * pre-emptible environemnt, or force a context switch otherwise
 * \param   host_h  Handle returned by qhalhost_open
 * \param   time_ms Time to wait in milliseconds
 * \return  QHAL_SUCCESS on success, see qhal_errno.h.
 */
int qhalhost_wait(qhalhost_handle_t host_h, unsigned long time_ms);

/**qhalhost_wait_ns
 * Wait for a specific time in ns
 * NOTE: This is an exact wait, it should be implemented to be as exact as possible
 * No context switch should be allowed in this wait
 * \param   host_h  Handle returned by qhalhost_open
 * \param   time_ns Time to wait in nanoseconds
 * \return  QHAL_SUCCESS on success, see qhal_errno.h.
 */
int qhalhost_wait_ns(qhalhost_handle_t host_h, unsigned long time_ns);

/**qhalhost_malloc
 * Allocate host memory for qhal use
 * \param   host_h  Handle returned by qhalhost_open
 * \param   size    Number of bytes to allocate
 * \return  NULL if error, pointer to the newly allocated memory otherwise
 */
void *qhalhost_malloc(qhalhost_handle_t host_h, int size);

/**qhalhost_free
 * Free meory allocated with qhalhost_malloc
 * \param   host_h  Handle returned by qhalhost_open
 * \param   ptr     Pointer returned by qhalhost_malloc
 * \return  QHAL_SUCCESS on success, see qhal_errno.h.
 */
int qhalhost_free(qhalhost_handle_t host_h, void *ptr);

/**qhalhost_lock
 * After this method succeeded, caller will be the only one to have access
 * to the interface used, call qhalhost_unlock to allow other user to access
 * host interface
 * \param   host_h  Handle returned by qhalhost_open
 * \return  QHAL_SUCCESS on success, see qhal_errno.h.
 */
int qhalhost_lock(qhalhost_handle_t host_h);

/**qhalhost_unlock
 * Will allow other user to access ressources again. YOU MUST NOT
 * FORGET TO CALL THIS OTHERWISE BUS WILL NOT BE DEAD
 * \param   host_h  Handle returned by qhalhost_open
 * \return  QHAL_SUCCESS on success, see qhal_errno.h.
 */
int qhalhost_unlock(qhalhost_handle_t host_h);

/**qhalhost_reset
 * Assert reset and configure the board
 * \param   host_h  Handle returned by qhalhost_open
 * \return  QHAL_SUCCESS on success, see qhal_errno.h.
 */
int qhalhost_reset(qhalhost_handle_t host_h);

/**qhalhost_close
 * Last call that will release hostbus ressource
 * \param   host_h  Handle returned by qhalqhalhost_open
 * \return  QHAL_SUCCESS on success, see qhal_errno.h.
 */
int qhalhost_close(qhalhost_handle_t host_h);

#ifdef    __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

#endif  /* ifndef __QHAL_HOST_H */
