/******************************************************************************** 
*
* The content of this file or document is CONFIDENTIAL and PROPRIETARY
* to Mobilygen Corporation.  It is subject to the terms of a
* License Agreement between Licensee and Mobilygen Corporation.
* restricting among other things, the use, reproduction, distribution
* and transfer.  Each of the embodiments, including this information and
* any derivative work shall retain this copyright notice.
* 
* Copyright 2008 Mobilygen Corporation
* All rights reserved.
* 
* QuArc is a registered trademark of Mobilygen Corporation.
* 
*******************************************************************************/

/*******************************************************************************
* $Id: readstreamer.h 23815 2010-02-24 18:44:27Z echung $
*******************************************************************************/

/// \defgroup rsgroup_a Readstreamer
/// \{

///////////////////////////////////////////////////////////////////////////////
///
/// \page readstreamer
/// \latexonly
/// \chapter{ReadStreamer}
/// \endlatexonly
/// 
/// The readstreamer is a host-only object that is responsible for sending
/// bitstreams to the codec.  The readstreamer manages a set of objects
/// called "bitstream readers" which are responsible for generating the
/// streams.  Typically these bitstream readers convert a media container
/// such as RTP, MP4 etc. from their native format to the codec's QBOX
/// interchange format.  The readstreamer can manage multiple instances of
/// bitstream readers at one time and thus allows for multichannel decoding.
///
/// The readstreamer is instantiated with the ReadStreamer_Create() C call.
/// or via the sysctl0:createReadStreamer() LUA function.  The writestreamer
/// object embeds a single thread which will be started with the create() command.
/// However, the thread will be put into an idle state until a start() or open()
/// command is issued.
///
/// Once started, bitstream readers can be opened or closed.  A reader will start
/// reading data from a file once play() is called.  The thread will keep running
/// until all the buffers are full.  Then, the thread will be suspended until
/// buffer space is available again.  When an end-of-file signal is returned
/// from a reader, the reader will be set to inactive unless there is another
/// file queued.  If there are no more active readers, the thread will return to 
/// idle state.
///
/// The readstreamer supports bitstream transfer via the codec's bitstream port
/// or through memory transfers.  At this time, MPEG2 and JPEG decode is only 
/// supported via the bitstream port whereas AVC decode is supported through
/// both.  The principal advantage of using direct memory transfer is that 
/// block transfer times are subject to a much smaller amount of jitter and
/// are not subject to being blocked for prolonged periods of time.
///
/// The readstreamer initiates transfers based on each decoder's input buffer 
/// being empty.  Also, the readstreamer prevents buffer overflow from happening.
/// However, the buffer management mechanisms are different under different 
/// transfer modes.  For bitstream port transfer mode, the readstreamer makes 
/// use of the underflow events from the AVDecoder objects to control the data 
/// flow.  The readstreamer first polls each AVDecoder object for buffer emptiness.
/// Once a buffer gets full, readstreamer will stop sending data to that particular
/// decoder.  Buffer emptiness polling will also be suspended at that time.  
/// Upon receipt of an underflow event from the decoder, the readstreamer will 
/// resume sending data and start polling again.  For direct memory transfer mode,
/// there is a shared memory block between the decoder and the readstreamer.  
/// All the buffer information can be found inside that block (e.g. 
/// read/write pointer).  When data is written to the buffer, the readstreamer 
/// will update its write pointer.  Similarly, when data is read from the buffer, 
/// the decoder update its read pointer.  Buffer emptiness can then be calculated
/// using the read/write pointers.
///
/// The readstreamer also has the ability to re-map stream IDs for each 
/// stream that is opened by a bitstream writer.  The mapping is specified by
/// when the bitstream reader is handled.  Furthermore, the readstreamer
/// supports files that contain multiple streams (including multiple video
/// and audio streams) that should be sent to multiple decoders without
/// opening and reading the source file more than once.
/// 
/// The readstreamer supports a simple form of forward I-frame only playback
/// where only every Nth I-frame is transferred.  The bitstream reader must
/// also support this mode for it to work.  At this time only the qboxreader
/// supports this feature.
///
/// Each bitstream reader is responsible for buffering its own data.  The
/// readstreamer contains no internal buffering and uses memory allocated
/// by the individual readers.  The readstreamer requests blocks from 
/// each reader and sends them to the matching AVDecoder when space becomes
/// available.  Note that this technique of pre-buffering data in memory
/// results in higher performance rather than requesting the data from the
/// reader only when space becomes available.

#ifndef __READSTREAMER_H
#define __READSTREAMER_H

#include <qhal_em.h>
//#include "avdecapi.h"
#include "bsreader.h"
#include "writestreamer.h"
#include "qbox.h"

typedef enum
{
    READSTREAMER_XFER_TYPE_VIDEO = 1,
    READSTREAMER_XFER_TYPE_AUDIO,
	READSTREAMER_XFER_TYPE_TS,
    READSTREAMER_XFER_TYPE_UNKNOWN
} READSTREAMER_XFER_TYPE;

#define MAX_BITSTREAMREADERS        16
#define MAX_STREAMS_PER_READER      32
#define MAX_DECODE_BUFFERS          32

#define READSTREAMER_OK             1
#define READSTREAMER_ERROR          0

// Configuration parameter values
/// \brief Used for open(), indicates that all video streams will be set to active.
#define Q_RS_CFP_VID_ID_ALL         65536
/// \brief Used for open(), indicates that all audio streams will be set to active.
#define Q_RS_CFP_AUD_ID_ALL         65537

/// \brief Enable or disable update messages displayed on event receipt from the codec
#define Q_RS_CFG_SHOW_UPDATE                    1

/// \brief Number of frames to buffer up for each stream before sending data to codec.  0 to disable.  Note that this only applies to bitstream interface mode
#define Q_RS_CFG_NUM_FRAMES_IN_GATHER_BUFFER    2

/// \brief Adjust the size of internal buffer used to write data to the codec
#define Q_RS_CFG_WRITE_BLOCKSIZE                3

/// \brief Set the SCHED_FIFO priority for this thread. Needs to be called before starting.
/// Set to 0 to disable. Disabled by default. POTENTIALLY DANGEROUS, KNOW WHAT YOU ARE DOING.
#define Q_RS_CFG_THREAD_PRIORITY                4




/// \brief Size of guardband in the video input buffer that is maintained by the readstreamer.  The buffer fullness should not exceed the buffer size minus the guardband.
#define Q_RS_CFG_BSR_GUARDBAND                  1

/// \brief This parameter enables looping in the specified bitstream reader.  The behavior is stream-specific but typical involves seeking to the start of the file and resending data.
#define Q_RS_CFG_BSR_LOOPMODE                   2

/// \brief This parameter enables bitstream transfer via direct DMA to memory rather than to the bitstream port when set to 1.
#define Q_RS_CFG_BSR_DMA_XFER_MODE              3

/// \brief When set to 1 this parameter forces the bitstream reader to not send audio data.  Note that not all bitstream readers support this feature.
#define Q_RS_CFG_BSR_SKIP_AUDIO                 4

/// \brief When set to 1 this parameter forces the bitstream reader to not send video data.  Note that not all bitstream readers support this feature.
#define Q_RS_CFG_BSR_SKIP_VIDEO                 5

/// \brief This parameter sets the current file position for the bitstream reader.  Note that not all bitstream readers support this feature.
#define Q_RS_CFG_BSR_SET_START_OFFSET           6

/// \brief This parameter sets the current speed for I-frame only playback.  For a value of N, only every Nth I-frame is transferred.
#define Q_RS_CFG_BSR_IFRAME_SCAN_SPEED          7

/// \brief This parameter sets non-blocking read operation in the bitstream reader.
#define Q_RS_CFG_BSR_NONBLOCK_READ              8

/// \brief This parameter sets the number of bytes that will be read from the bitstream reader. 0 means read till EOF.
#define Q_RS_CFG_BSR_SET_PLAY_LENGTH            9

/// \brief This parameter sets the playback options.
#define Q_RS_CFG_BSR_SET_PLAY_OPTIONS           10

/// \brief This parameter sets the starting PTS of the stream, all packets with PTS prior to it will be dropped.
#define Q_RS_CFG_BSR_SET_START_PTS              11

/// \brief This parameter sets the last PTS of the stream, all packets with PTS after it will be dropped.  Mainly used for reverse trick play.
#define Q_RS_CFG_BSR_SET_LAST_PTS               12

/// \brief Ioctl command to get the current file position from bitstream reader
#define Q_RS_IOCTL_GET_FILE_POSITION            1

/// \brief Ioctl command to reset the stream IDs for bitstream reader
#define Q_RS_IOCTL_SET_STREAM_ID                2

/// \brief Ioctl command to see if input data is ready to be sent from the bitstream reader
#define Q_RS_IOCTL_GET_DATA_READINESS           3

/// \brief Ioctl command to see the state of the plugin; used for watchdog monitoring. Possible values: "RUNNING", "NO_NEW_REQ" and "NO_INP_DATA"
#define Q_RS_IOCTL_GET_WD_STATS                30

/// \brief Ioctl command to change host/port for UDPTSReader plugin
#define Q_RS_CHANGE_HOST                       31

/// \brief BSR notifying readstreamer input data is ready
#define Q_BSR_CALLBACK_DATA_READY               1

typedef void *READSTREAMER_HANDLE;

typedef struct
{
    int  id;
    char state[256];
    int  curBitrate;
    int  avgBitrate;
    int  usage;
    int  getblkusage;
    int  pollusage;
    int  xferusage;
    char bsrInfo[MAX_BITSTREAMREADERS][256];
    char bufInfo[MAX_DECODE_BUFFERS][256];
} READSTREAMER_QUERY_DATA;

typedef struct
{
    int nStreams;
    int srcId[MAX_STREAMS_PER_READER];
    int codecId[MAX_STREAMS_PER_READER];
} STRUCT_RS_IOCTL_SET_STREAM_ID;

/// \brief C Constructor used to create a readstreamer object.  The
/// embedded thread is instantiated but kept in a paused state until
/// the Start() function is called.
/// \return A handle to the newly created object.
//READSTREAMER_HANDLE ReadStreamer_Create(int codec, int mempio, SYSCONTROL_HANDLE hsys);

/// \brief This function is not supported at this time.
void    ReadStreamer_Terminate(READSTREAMER_HANDLE h);

/// \brief Starts the embedded thread in the readstreamer object.  The
/// thread starts looking for bitstream readers with work to do.
/// \param h Handle to a readstreamer object
/// \return 1 for Success, 0 for Failure
int     ReadStreamer_Start(READSTREAMER_HANDLE h);

/// \brief This function is not supported at this time.
int     ReadStreamer_Stop(READSTREAMER_HANDLE h);

/// \brief This function is a stub that always returns success at
/// this time.  It is here for future expansion.
/// \param h Handle to a readstreamer object.
int     ReadStreamer_StopAndFlush(READSTREAMER_HANDLE h);

/// \brief This function configures an already opened bitstream reader.  The effect of 
/// each parameter is reader-specific.  
/// \param h Handle to a readstreamer object
/// \param idx Index of an opened bitstream reader
/// \param cmd Ioctl command
/// \param arg String value to pass 
/// \return 0 for failure, 1 for success
int ReadStreamer_Ioctl (READSTREAMER_HANDLE h, int idx, int cmd, void *arg);

/// \brief This function opens a new bitstream reader. Assuming the readstramer thread is running,
/// the reader will be checked against its matching AVDecoders for available data to send.  
/// The function returns an integer which is subsequently used to reference this opened reader.
/// \param h Handle to a bitstream reader
/// \param type Pointer to a string that defines the type of bitstream reader to instantiate and open
/// \param filename Pointer to a filename which is passed to the bitstream reader. The filename
/// can either be a simple string or comma-separated arguments.  The arguments are reader specific.
/// \param srcId A pointer to an array of stream Ids that are embedded in the original QBOX stream.  
/// Id's found in this array will be considered active and sent to codec.  The readstreamer will map 
/// these source stream Ids to a matching set of stream Id's (see codecId).
/// \param codecId A pointer to an array of stream Id's to which the srcId's are mapped.  Ids are
/// mapped by index position in the array.
/// \param nStreams The number of entries in the srcId and codecId arrays.
/// \param startOfst The starting offset (in bytes) in the file where streaming should begin.
/// \param playLength Number of bytes the reader will send to codec.  0 means play till EOF.
/// \param flags Special options for the reader.
/// \return idx Index of the opened bitstream reader.  -1 for failure.
///
/// The ReadStreamer_OpenBitstreamReadrByType function opens a bitstream reader type as specified 
/// by the argument type.  The readstreamer object maintains a translation table between 
/// type as expressed as a string and a true object.  The external API uses string to specify 
/// type which is subsequently translated to a true object.
///
/// Filename is bitstream reader type-specific.  For example, most storage readers (meaning those 
/// that open files and read a stream to the file) use the string as the full path to the 
/// destination file.  However, many bitstream readers have optional or mandatory arguments that 
/// must be specified.  The convention is to specify arguments as a comma-separated list.  For 
/// example, a reader may take the argument "fn" which specifies filename and "bs" which 
/// specifies buffer size. The reader would then be opened with the
/// filename "fn=/path/to/file,bs=1024".
int     ReadStreamer_OpenBitstreamReaderByType(READSTREAMER_HANDLE h, char *type, char *filename, int *srcId, int *codecId, 
                                               int nStreams, unsigned long long startOfst, unsigned int playLength, unsigned int flags);

/// \brief This function marks an already-opened bitstream reader as in the play state.  The
/// readstreamer's embedded thread will now request data from the reader and send it to the codec.
/// \param h Handle to a readstreamer
/// \param idx Index of the opened bitstreamreader
/// \return 1 for Success, 0 for Failure
int     ReadStreamer_PlayBitstreamReader(READSTREAMER_HANDLE h, int idx);

/// \brief This function calls the close function for the specified bitstream reader and
/// removes the reader from the play state so it is no longer checked for data.  The
/// functionality of close is bitstream reader specific.
/// \param h Handle to a bitstream reader
/// \param idx Index to a an opened bitstream reader
int     ReadStreamer_CloseBitstreamReader(READSTREAMER_HANDLE h, int idx);

/// \brief This function sets single buffered configuration parameters of the readstreamer
/// itself.  Note that this function is not used to configure individual bitstream readers
/// that have already been opened.  Parameters available for configuration are
/// Q_RS_CFG_*.
/// \param h Handle to a bitstream reader
/// \param param Parameter index to configure
/// \param value Parameter value to set
/// \return 1 for Success, 0 for Failure
int     ReadStreamer_Configure(READSTREAMER_HANDLE h, int param, int value);

/// \brief This function configures an already opened bitstream reader.  The effect of 
/// each parameter is reader-specific.  Parameters available for configuration are
/// Q_RS_CFG_BSR_*.
/// \param h Handle to a bitstream reader
/// \param idx Index of an opened bitstream reader
/// \param param Parameter name to configure
/// \param value Value to which the parameter is set
/// \return 1 for Success, 0 for Failure
int     ReadStreamer_ConfigureBSR(READSTREAMER_HANDLE h, int idx, int param, long long value);

/// \brief This function is used to bind a specified AVDecoder object to the readstreamer
/// so that the readstreamer can handle its buffer overflow and underflow events.
/// \param h Handle to a readstreamer object
/// \param dec Pointer to a AV Decoder object
/// \return 1 for Success, 0 for failure
//int     ReadStreamer_BindDecoder(READSTREAMER_HANDLE h, AVDEC_HANDLE dec);

/// \brief This function unbinds a decoder from the readstreamer.  The readstreamer will
/// no longer request data requests from this decoder.
/// \param h Handle to a bitstream reader
/// \param dec Pointer to an AVDecoder object
/// \return 1 for Success, 0 for failure
//int     ReadStreamer_UnbindDecoder(READSTREAMER_HANDLE h, AVDEC_HANDLE dec);

/// \brief This function returns the current file position of the specified bitstream reader.
/// \param h Handle to a readstreamer
/// \param idx Index of an opened reader
/// \return 1 for Success, 0 for failure
int     ReadStreamer_GetFilePosition(READSTREAMER_HANDLE h, int idx);

/// \brief This function queues input files to be opened by the specified bitstream reader
/// immediately upon completion of the current file.  This call queues the specified filename
/// which is then opened as soon as the bitstream reader returns end-of-file.
/// \param h Handle of a readstreamer
/// \param idx Index of a bitstream reader object
/// \param filename Name of the queued file
/// \param srcId A pointer to an array of stream Ids that are embedded in the original QBOX stream.  
/// Id's found in this array will be considered active and set the codec.  The readstreamer will map 
/// these source stream Ids to a matching set of stream Id's (see codecId).
/// \param codecId A pointer to an array of stream Id's to which the srcId's are mapped.  Ids are
/// mapped by index position in the array.
/// \param nStreams The number of entries in the srcId and codecId arrays.
/// \param startOfst The starting offset (in bytes) in the file where streaming should begin.
/// \param playLength Number of bytes the reader will send to codec.  0 means play till EOF.
/// \param flags Special options for the reader.
/// \return Number of available slots in the queue
int     ReadStreamer_QueueFile(READSTREAMER_HANDLE h, int idx, char *filename, int *srcId, int *codecId, int nStreams,
                               unsigned long long startOfst, unsigned int playLength, unsigned int flags);

/// \brief This function adds a bitstream readers to the list of available readers
/// \param name Name of bitstream reader type 
/// \param bsr Handle to bitstream reader 
/// \return 1 for success, 0 for failure
int     ReadStreamer_AddBitstreamReaderTypeByName(char *name, BITSTREAMREADER *bsr);

/// \brief This function resets the block arrival time stats in the query information
/// \param h Handle to a readstramer object
/// \param idx Index of a bitstream reader object
/// \return 1 for success, 0 for failure
int     ReadStreamer_ResetBlkArrivalStats(READSTREAMER_HANDLE h, int idx);

/// \brief This function resets the active stream IDs for bitstream reader 'idx'
/// \param h Handle to a readstramer object
/// \param idx Index of a bitstream reader object
/// \param srcId (See above for definition)
/// \param codecId (See above for definition)
/// \param nStreams Number of entries in the srcId and codecId arrays
int     ReadStreamer_SetStreamID(READSTREAMER_HANDLE h, int idx, int *srcId, int *codecId, int nStreams);

/// \brief This function displays useful information regarding the state of the readstreamer. 
/// \param h Handle to a readstramer object
/// \param data Query data stored inside here
/// \param silent Does not display query information when silent is set to 1
/// \return 1 for success, 0 for failure
int     ReadStreamer_Query(READSTREAMER_HANDLE h, READSTREAMER_QUERY_DATA *data, int silent);

int     ReadStreamer_SetReaderCallback(READSTREAMER_HANDLE h, int idx, BITSTREAMREADER_CALLBACK fn, void *arg);


#endif /* __READSTREAMER_H */
