
#ifndef _RTMP_QUEUE_H
#define _RTMP_QUEUE_H

#include <sys/un.h>
#include <netinet/tcp.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <sys/timeb.h>
#include <sys/time.h>

#include "semaphore.h"

#ifdef RTSP_PRESENT
#include "BasicTypes.h"
#endif
#include "ServerInclude.h"
#include "ServerBasicTypes.h"
#include "ServerCommonDefines.h"

enum {
    Q_DBG_LEVEL_0,
    Q_DBG_LEVEL_1,
    Q_DBG_LEVEL_2,
    Q_DBG_LEVEL_3,
    Q_DBG_LEVEL_4,
    Q_DBG_LEVEL_INFO
};

#define RTMP_QUEUE_DEBUG_LEVEL Q_DBG_LEVEL_INFO

/* Macro for debugging */
#define RTMP_QUEUE_PRINT(dbgLevel, format, ...) \
        if(dbgLevel >= RTMP_QUEUE_DEBUG_LEVEL) \
            fprintf (stdout, "RTMPWriter: QUEUE: %s: " format, __FUNCTION__, ## __VA_ARGS__)

typedef struct _RtmpQueueBWMItem        RtmpQueueBWMItem;
typedef struct _RtmpQueueBWM            RtmpQueueBWM;

typedef struct _RtmpQueueBufferItem     RtmpQueueBufferItem;
typedef struct _RtmpQueueBuffer         RtmpQueueBuffer;

typedef struct _RtmpQueueDebug          RtmpQueueDebug;

typedef struct _RtmpQueueAdaptiveBitrate RtmpQueueAdaptiveBitrate;

typedef struct _RtmpQueue               RtmpQueue;

// Queue Buffer return statuses
enum {
    RTMP_QUEUE_BUFFER_OK,
    RTMP_QUEUE_BUFFER_EPTR,
    RTMP_QUEUE_BUFFER_FULL,
    RTMP_QUEUE_BUFFER_EMPTY,
    RTMP_QUEUE_BUFFER_ERROR
};

// Queue BWM return statuses
enum {
    RTMP_QUEUE_BWM_OK,
    RTMP_QUEUE_BWM_EPTR,
    RTMP_QUEUE_BWM_ERROR
};

// Queue Debug return statuses
enum {
    RTMP_QUEUE_DEBUG_OK,
    RTMP_QUEUE_DEBUG_ERROR
};

// Queue Adaptive Bitrate return statuses
enum {
    RTMP_QUEUE_ADAPTIVE_BITRATE_OK,
    RTMP_QUEUE_ADAPTIVE_BITRATE_EPTR,
    RTMP_QUEUE_ADAPTIVE_BITRATE_ERROR
};

struct _RtmpQueueBWMItem
{
    struct timeval  time;
    INT32           dataSize;
};

struct _RtmpQueueBWM
{
    INT32               maxSize;
    RtmpQueueBWMItem    *queue;

    INT32               size;
    INT32               head;
    INT32               tail;

    UINT64              total;
    UINT64              average;
    INT32               bitrate;

    struct timeval      lastUpdate;
    struct timeval      lastMsg;
    struct timeval      firstUptime;
    struct timeval      lastUptime;
};

struct _RtmpQueueBufferItem
{
    RTMP_EXACT_MSG_TYPE type; // data type (audio/video)

    UINT8               *dataBuff; // Pointer to audio/video data
    INT32               dataSize; // Size of audio/video data
    INT32               deviceFd; // Zero copy device fd
    void                *vZeroPtr; // Zero copy pointer

    UINT64              timeStamp; // Timestamp from the encoder
};

struct _RtmpQueueBuffer
{
    sem_t               emptySem;
    pthread_mutex_t     frameMutex;

    INT32               maxSize;
    RtmpQueueBufferItem *queue;

    INT32               size;
    INT32               head;
    INT32               tail;
};

struct _RtmpQueueDebug
{
    FILE                *fp;
    UINT64              first_pts;
    INT32               frameCount;
    UINT64              uptime;
    INT32               dropCount;
    INT32               qp;

    UINT8               file;
    UINT8               burnin;

    UINT8               path[MEDIUM_BUFFER_SIZE];
    INT32               intervalFrames;
};

struct _RtmpQueueAdaptiveBitrate
{
    INT32   limit;
    INT32   current;
    UINT64  lastIncreaseTimestamp;
    UINT64  lastDecreaseTimestamp;
    UINT8   warningIssued;

    INT32   min;

    UINT64  startTimestamp;
    INT32   settleTime;

    INT32   recoverThreshold;
    INT32   recoverStepPercent;
    INT32   recoverMinStep;
    INT32   recoverInterval;

    INT32   adjustThreshold;
    INT32   adjustPercent;

    INT32   triggerThreshold;
    INT32   decreasePercent;
    INT32   decreaseInterval;

    INT32   margin;

    HANDLE  logPtr;
};

struct _RtmpQueue
{
    SHANDLE rtmpSession;
    void *avcencHandle;

    RtmpQueueBuffer     *audioBuffer;
    RtmpQueueBuffer     *videoBuffer;

    // Bandwidth monitor
    RtmpQueueBWM        *bwMonitor;

    // Debug
    RtmpQueueDebug      *debug;

    // Adaptive Bitrate
    RtmpQueueAdaptiveBitrate *adapt;

    UINT64  prevAudioTimeStamp;
    UINT64  prevVideoTimeStamp;

    UINT64  lastSentVideoTimeStamp;
    UINT64  lastSentAudioTimeStamp;

    int senderFault;

    UINT8               threadStarted;
    pthread_t           ThreadId;
};


// Buffer item functions
int RtmpQueueBufferItemReset(RtmpQueueBufferItem *item);
int RtmpQueueBufferItemCopy(RtmpQueueBufferItem *to, RtmpQueueBufferItem *from);

// Buffer functions
void* RtmpQueueBufferCreate(int bufferSize);
int RtmpQueueBufferReset(RtmpQueueBuffer *buffer);
int RtmpQueueBufferDestroy(RtmpQueueBuffer *buffer);
int RtmpQueueBufferAdd(RtmpQueueBuffer *buffer,
        RTMP_EXACT_MSG_TYPE type, UINT8 *dataBuff, INT32 dataSize,
        INT32 deviceFd, void *vZeroPtr, UINT64 timeStamp);
int RtmpQueueBufferRemove(RtmpQueueBuffer *buffer);

RtmpQueueBufferItem* RtmpQueueBufferTake(RtmpQueueBuffer *buffer);
int RtmpQueueBufferRelease(RtmpQueueBuffer *buffer);
int RtmpQueueBufferFlush(RtmpQueueBuffer *buffer);


// BWM Item Functions
int RtmpQueueBWMItemReset(RtmpQueueBWMItem *item);

// BWM Functions
void *RtmpQueueBWMCreate(int monitorSize);
int RtmpQueueBWMReset(RtmpQueueBWM *m);
int RtmpQueueBWMDestroy(RtmpQueueBWM *monitor);
int RtmpQueueBWMAdd(void *m, void *avcencHandle, INT32 size);


// Debug Functions
void *RtmpQueueDebugCreate(int burninEnable, int fileEnable, char *filePath, int interval);
int RtmpQueueDebugReset(RtmpQueueDebug *monitor);
int RtmpQueueDebugDestroy(RtmpQueueDebug *monitor);
int RtmpQueueDebugInit(RtmpQueueDebug *monitor, UINT64 first_pts);
int RtmpQueueDebugUpdate(RtmpQueueDebug *monitor, void *avcencHandle, UINT64 pts);
int RtmpQueueDebugBurnin(RtmpQueueDebug *monitor, void *avcencHandle, UINT64 fullness, int bitrate, int target);
int RtmpQueueDebugLog(RtmpQueue *queue, UINT64 pts, UINT64 fullness, struct timeval *start, struct timeval *end);

// Adaptive Bitrate Functions
void *RtmpQueueAdaptiveBitrateCreate(HANDLE log,
                                    INT32 adapt_min,
                                    INT32 adapt_settleTime,
                                    INT32 adapt_recoverThreshold,
                                    INT32 adapt_recoverStepPercent,
                                    INT32 adapt_recoverMinStep,
                                    INT32 adapt_recoverInterval,
                                    INT32 adapt_adjustThreshold,
                                    INT32 adapt_adjustPercent,
                                    INT32 adapt_triggerThreshold,
                                    INT32 adapt_decreasePercent,
                                    INT32 adapt_decreaseInterval,
                                    INT32 adapt_margin);
int RtmpQueueAdaptiveBitrateReset(RtmpQueueAdaptiveBitrate *abr);
int RtmpQueueAdaptiveBitrateInit(RtmpQueueAdaptiveBitrate *abr, INT32 bitrate);
int RtmpQueueAdaptiveBitrateUpdate(RtmpQueueAdaptiveBitrate *abr, INT32 measuredBitrate, INT32 currentSize, INT32 maxSize, void *avcencHandle, UINT64 timeStamp, RtmpQueueDebug *rtmpDebug);
int RtmpQueueAdaptiveBitrateDestroy(RtmpQueueAdaptiveBitrate *abr);

void *RtmpSenderThread(void *arg);

#endif  /* _RTMP_QUEUE_H */
