/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

/*******************************************************************************
 *
 * File Name    :   RtmpServerApi.h
 *
 * Description  :   This file contains API declarations, enums, structures
 *                  which are exported to application
 *
 * History:
 *  MAY/13/2010, ARJ,   Created the file.
 *  MAY/14/2010, PRP,   Started RTMP implementation
 ******************************************************************************/

#ifndef _INCLUDE_RTMPSERVERAPI_H
#define _INCLUDE_RTMPSERVERAPI_H

/*********************
 * Include Files
 *********************/
#ifdef RTSP_PRESENT
#include "BasicTypes.h"
#endif
#include "ServerInclude.h"
#include "ServerBasicTypes.h"
#include "ServerCommonDefines.h"

/*********************
 * Defines and Macros
 *********************/
#define RTMP_SUCCESS                0       /* Success                        */
#define RTMP_FAIL                   -1      /* Failure                        */

/*********************
 * Typedefs and Enums
 *********************/
typedef enum _mErrorCodes
{
    RTMP_INVALID_INPUT_VALUE = -99,         /* Invalid input argument         */
    RTMP_INVALID_SERVER_HANDLE,             /* Invalid server handle          */
    RTMP_INVALID_SESSION_STATE,             /* Invalid session state          */
    RTMP_INVALID_CLIENT_STATE,              /* Invalid client state           */
    RTMP_AUDIO_CODEC_NOT_SUPPORTED,         /* Unsupported audio codec        */
    RTMP_VIDEO_CODEC_NOT_SUPPORTED,         /* Unsupported video codec        */
    RTMP_AUDIO_INFO_ADDITION_FAILED,        /* Failed to add audio information*/
    RTMP_VIDEO_INFO_ADDITION_FAILED,        /* Failed to add video information*/
    RTMP_METADATA_GENERATION_FAILED,        /* Generation of metadata failed  */
    RTMP_NETWORK_INTERFACE_NOT_FOUND,       /* Network interface not present  */
    RTMP_MEMORY_ALLOCATION_FAILED,          /* Memory allocation failed       */
    RTMP_SESSION_NAME_EXISTS,               /* Session name already exists    */
    RTMP_SOCKET_CREATION_FAILED,            /* Socket creation failed         */
    RTMP_SOCKET_BIND_FAILED,                /* Binding socket failed          */
    RTMP_SOCKET_LISTEN_FAILED,              /* Listening to socket failed     */
    RTMP_THREAD_CREATION_FAILED,            /* Creation of thread failed      */
    RTMP_WAIT_FOR_THREAD_FAILED,            /* Waiting for thread failed      */
    RTMP_NO_CLIENT_CONNECTED,               /* No Rtmp client is connected    */
    RTMP_DATA_SEND_SOCK_ERROR,              /* Error in sending data          */
    RTMP_DATA_RECV_SOCK_ERROR,              /* Error in receiving data        */
    RTMP_MUTEX_LOCK_CREATION_FAILED,        /* Lock creation failed           */
    RTMP_MUTEX_LOCK_FAILED,                 /* Locking failed                 */
    RTMP_MUTEX_UNLOCK_FAILED,               /* Unlocking failed               */
    RTMP_LOCK_DESTROY_FAILED,               /* Destroying lock failed         */
    RTMP_TIMER_NOT_INITIALIZED,             /* Timer is not initialized       */
    RTMP_SEND_PACKET_FAILED,                /* Sending packet failed          */
    RTMP_GET_IP_ADDR_FAILED                 /* Failed in getting ip address   */

} mErrorCodes;

typedef enum _STREAM_TYPE
{
    VIDEO_STREAM,                           /* Data stream type is video      */
    AUDIO_STREAM                            /* Data stream type is audio      */

}STREAM_TYPE;

typedef enum _VIDEO_CODEC_TYPE
{
    H264_VIDEO_CODEC = 0x07,                /* Video codec is H.264           */
    INVALID_VIDEO_CODEC                     /* Invalid video codec            */

} VIDEO_CODEC_TYPE;

typedef enum _AUDIO_CODEC_TYPE
{
    AAC_AUDIO_CODEC = 0x0a,                 /* Audio codec is AAC             */
    INVALID_AUDIO_CODEC                     /* Invalid audio codec            */

} AUDIO_CODEC_TYPE;

/*********************
 * Structures
 *********************/
typedef struct _RtmpVideoInfo
{
    VIDEO_CODEC_TYPE    VdCodecType;        /* Type of video codec            */
    INT32               VdBitrate;          /* Video bitrate                  */
    INT32               VdFr1001;           /* Fractional framerate flag      */
    INT32               VdFr;               /* Video framerate                */
    INT32               VdKfi;               /* Video key frame freq          */
    INT32               VdConfigDataSize;   /* Size of video config data      */
    UINT8               *VdConfigData;      /* Pointer to video config data
                                               containing SPS, PPS along with
                                               its size instead of start code */
} RtmpVideoInfo;

typedef struct _RtmpAudioInfo
{
    AUDIO_CODEC_TYPE    AdCodecType;        /* Type of audio codec            */
    INT32               AdBitrate;          /* Audio bitrate                  */
    INT32               AdConfigDataSize;   /* Size of audio config data      */
    UINT8               *AdConfigData;      /* Pointer to audio config data   */

} RtmpAudioInfo;

/************************
 * Function declarations
 ************************/

/* API to initialize the RTMP server stack                                    */
RTMPSERVERLIB SHANDLE RtmpServerInit (INT32 *errorCode);

/* API to deinitialize the RTMP server stack                                  */
INT32 RtmpServerDeInit (SHANDLE libHandle);

/* API to create RTMP server                                                  */
RTMPSERVERLIB SHANDLE RtmpServerCreate (HANDLE LOG_HANDLE,
                                        UINT8 *nwInterface,
                                        UINT16 rtmpPort, UINT16 rtmptPort, UINT8 dscp, INT32 stocsize,
                                        INT32 *errorCode);

/* API to destroy RTMP server                                                 */
RTMPSERVERLIB INT32 RtmpServerDestroy (SHANDLE serverHandle);

/* API to create RTMP session                                                 */
RTMPSERVERLIB SHANDLE RtmpSessionCreate (SHANDLE serverHandle,
                                         UINT8 *sessionName,
                                         INT32 *errorCode);

RTMPSERVERLIB INT32 RtmpSenderSettings( SHANDLE sessionHandle,
                                        PublishSenderType senderType,
                                        INT32 tick_120,
                                        UINT8 dscp,
                                        UINT32 avcencHandle,
                                        INT32 sender_audioBufferSize,
                                        INT32 sender_videoBufferSize);

RTMPSERVERLIB INT32 RtmpBWMSettings( SHANDLE sessionHandle,
                                    UINT8 bwmEnable,
                                    INT32 bwm_size);

RTMPSERVERLIB INT32 RtmpDebugSettings(SHANDLE sessionHandle,
                                    UINT8 debugEnable,
                                    UINT8 dbg_burnin,
                                    UINT8 dbg_file,
                                    UINT8 *dbg_path,
                                    INT32 dbg_interval_frames);

RTMPSERVERLIB INT32 RtmpAdaptiveBitrateSettings(SHANDLE sessionHandle,
                                    INT32 adaptiveBitrate,
                                    INT32 adapt_currentBitrate,
                                    INT32 adapt_max,
                                    INT32 adapt_min,
                                    INT32 adapt_settleTime,
                                    INT32 adapt_recoverThreshold,
                                    INT32 adapt_recoverStepPercent,
                                    INT32 adapt_recoverMinStep,
                                    INT32 adapt_recoverInterval,
                                    INT32 adapt_adjustThreshold,
                                    INT32 adapt_adjustPercent,
                                    INT32 adapt_triggerThreshold,
                                    INT32 adapt_decreasePercent,
                                    INT32 adapt_decreaseInterval,
                                    INT32 adapt_margin);

/* API to destroy RTMP session                                                */
RTMPSERVERLIB INT32 RtmpSessionDestroy (SHANDLE sessionHandle);

/* API to add audio/video media information in RTMP session                   */
RTMPSERVERLIB INT32 RtmpAddMediaInfo (SHANDLE sessionHandle,
                                      RtmpVideoInfo *vdInfo,
                                      RtmpAudioInfo *adInfo);

RTMPSERVERLIB INT32 RtmpSendData (SHANDLE sessionHandle, UINT8 *dataBuff,
                    INT32 dataSize, INT32 deviceFd, VOID *vZeroPtr,
                    UINT64 timeStamp, STREAM_TYPE strmType);



/* Buffer Common */
#define MAX_MONITOR_ENTRIES 100 // Combined audio video
#define RTMP_QUEUE_HIGH_WATER_WAIT 6

extern void addPacketToMonitor(void *monitor, void *statusHandle, STREAM_TYPE type, struct timeval tv, INT32 size);
extern int timeval_subtract (struct timeval *result, struct timeval *x, struct timeval *y);



/* RTMP Monitor */
RTMPSERVERLIB INT32 RtmpMonitorInit(SHANDLE sessionHandle, UINT32 avcencHandle, INT32 bwMonitor);
RTMPSERVERLIB INT32 RtmpMonitorClear(SHANDLE sessionHandle);
RTMPSERVERLIB INT32 RtmpMonitorDestroy(SHANDLE sessionHandle);
RTMPSERVERLIB INT32 RtmpMonitorSendData (SHANDLE sessionHandle, UINT8 *dataBuff,
                    INT32 dataSize, INT32 deviceFd, VOID *vZeroPtr,
                    UINT64 timeStamp, STREAM_TYPE strmType);
RTMPSERVERLIB INT32 RtmpMonitorDebugInit(SHANDLE sessionHandle,
                                    UINT8 debugEnable,
                                    UINT8 dbg_burnin,
                                    UINT8 dbg_file,
                                    UINT8 *dbg_path,
                                    INT32 dbg_interval_frames);
RTMPSERVERLIB INT32 RtmpMonitorAdaptiveBitrateInit(SHANDLE sessionHandle,
                                    INT32 adaptiveBitrate,
                                    INT32 adapt_currentBitrate,
                                    INT32 adapt_max,
                                    INT32 adapt_min,
                                    INT32 adapt_recoverThreshold,
                                    INT32 adapt_recoverStepPercent,
                                    INT32 adapt_recoverMinStep,
                                    INT32 adapt_recoverInterval,
                                    INT32 adapt_adjustThreshold,
                                    INT32 adapt_adjustPercent,
                                    INT32 adapt_triggerThreshold,
                                    INT32 adapt_decreasePercent,
                                    INT32 adapt_decreaseInterval,
                                    INT32 adapt_margin);

/* Simple Buffer */
RTMPSERVERLIB INT32 RtmpSimpleBufferInit(SHANDLE sessionHandle);
RTMPSERVERLIB INT32 RtmpSimpleBufferSendData (SHANDLE sessionHandle, UINT8 *dataBuff,
                    INT32 dataSize, INT32 deviceFd, VOID *vZeroPtr,
                    UINT64 timeStamp, STREAM_TYPE strmType);
RTMPSERVERLIB INT32 RtmpSimpleBufferClear(SHANDLE sessionHandle);


/* RTMP Queue (Threaded) */
RTMPSERVERLIB INT32 RtmpThreadedQueueInit(SHANDLE sessionHandle, int audioBufferSize, int videoBufferSize);
RTMPSERVERLIB INT32 RtmpThreadedQueueReset(SHANDLE sessionHandle);
RTMPSERVERLIB INT32 RtmpThreadedQueueClose(SHANDLE sessionHandle);
RTMPSERVERLIB INT32 RtmpThreadedQueueDestroy(SHANDLE sessionHandle);
RTMPSERVERLIB INT32 RtmpThreadedQueueSendData(SHANDLE sessionHandle, UINT8 *dataBuff,
                                INT32 dataSize, INT32 deviceFd, VOID *vZeroPtr,
                                UINT64 timeStamp, STREAM_TYPE strmType);
RTMPSERVERLIB INT32 RtmpThreadedQueueBWMInit( SHANDLE sessionHandle,
                                    UINT8 bwmEnable,
                                    INT32 bwm_size);
RTMPSERVERLIB INT32 RtmpThreadedQueueDebugInit(SHANDLE sessionHandle,
                                    UINT8 debugEnable,
                                    UINT8 dbg_burnin,
                                    UINT8 dbg_file,
                                    UINT8 *dbg_path,
                                    INT32 dbg_interval_frames);
RTMPSERVERLIB INT32 RtmpThreadedQueueAdaptiveBitrateInit(SHANDLE sessionHandle,
                                    INT32 adaptiveBitrate,
                                    INT32 adapt_min,
                                    INT32 adapt_settleTime,
                                    INT32 adapt_recoverThreshold,
                                    INT32 adapt_recoverStepPercent,
                                    INT32 adapt_recoverMinStep,
                                    INT32 adapt_recoverInterval,
                                    INT32 adapt_adjustThreshold,
                                    INT32 adapt_adjustPercent,
                                    INT32 adapt_triggerThreshold,
                                    INT32 adapt_decreasePercent,
                                    INT32 adapt_decreaseInterval,
                                    INT32 adapt_margin);

/* Sender Queue Buffer */
RTMPSERVERLIB INT32 RtmpSenderInit(SHANDLE sessionHandle);
RTMPSERVERLIB INT32 RtmpSenderDestroy(SHANDLE sessionHandle);
RTMPSERVERLIB INT32 RtmpSenderClear(SHANDLE sessionHandle);
RTMPSERVERLIB INT32 RtmpSenderSendData (SHANDLE sessionHandle, UINT8 *dataBuff,
                    INT32 dataSize, INT32 deviceFd, VOID *vZeroPtr,
                    UINT64 timeStamp, STREAM_TYPE strmType);

#ifdef SPUTNIK_IOS

typedef void (* rtmpServerApi_set_monitoredbBitrate_callback_func)(int monitoredBitrate);
extern rtmpServerApi_set_monitoredbBitrate_callback_func rtmpServerApiSetMonitoredBitrateFunc;

//typedef int (* rtmpServerApi_get_bitrate_callback_func)(void);
//extern rtmpServerApi_get_bitrate_callback_func rtmpServerApiGetBitrateFunc;

typedef void (* rtmpServerApi_set_bitrate_callback_func)(int bitrate);
extern rtmpServerApi_set_bitrate_callback_func rtmpServerApiSetBitrateFunc;

void RTMPSASetBitrateCallbackFunction(rtmpServerApi_set_bitrate_callback_func func);


typedef UINT64 (* rtmpServerApi_get_currentPts_callback_func)(void);
extern rtmpServerApi_get_currentPts_callback_func rtmpServerApiGetCurrentPtsFunc;

void RTMPSAGetCurrentPtsCallbackFunction(rtmpServerApi_get_currentPts_callback_func func);

#endif

#endif  /* _INCLUDE_RTMPSERVERAPI_H */

/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * This file may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/
