/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * File may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/

/*******************************************************************************
 *
 * File Name    :   ServerAmf.h
 *
 * Description  :   File contains all defines, enums, structures,
 *                  functions declaration used in AMF module
 *
 * History:
 *  MAY/13/2010, ARJ,   Created the file.
 *  MAY/14/2010, PRP,   Started RTMP implementation
 ******************************************************************************/

#ifndef _INCLUDE_SERVERAMF_H
#define _INCLUDE_SERVERAMF_H

/*********************
 * Include Files
 *********************/
#include "ServerBasicTypes.h"

/*********************
 * Defines and Macros
 *********************/
#define AMF0_OBJ_ENCODING   0x00        /* Object encoding is of AMF0 type    */
#define AMF3_OBJ_ENCODING   0x03        /* Object encoding is of AMF3 type    */

/* Compare string object                                                      */
#define CMP_STR_OBJ(strObj1, strObj2) (((strObj1)->StrLen==(strObj2)->StrLen)\
                                            && \
                                        (!memcmp((strObj1)->StrData, \
                                                 (strObj2)->StrData, \
                                                 (strObj1)->StrLen)))

/*********************
 * Typedefs and Enums
 *********************/
typedef enum _AMF0_DATA_TYPE
{
    AMF0_NUMBER_MARKER          = 0x00, /* Marker Type - Number               */
    AMF0_BOOLEAN_MARKER         = 0x01, /* Marker Type - Boolean              */
    AMF0_STRING_MARKER          = 0x02, /* Marker Type - String               */
    AMF0_OBJECT_MARKER          = 0x03, /* Marker Type - Object               */
    AMF0_MOVIECLIP_MARKER       = 0x04, /* Marker Type - Movie clip           */
    AMF0_NULL_MARKER            = 0x05, /* Marker Type - Null                 */
    AMF0_UNDEFINED_MARKER       = 0x06, /* Marker Type - Undefined            */
    AMF0_REFERENCE_MARKER       = 0x07, /* Marker Type - Reference            */
    AMF0_ECMA_ARRAY_MARKER      = 0x08, /* Marker Type - ECMA array           */
    AMF0_EOF_OBJECT_MARKER      = 0x09, /* Marker Type - EOF object           */
    AMF0_STRICT_ARRAY_MARKER    = 0x0A, /* Marker Type - Strict array         */
    AMF0_DATE_MARKER            = 0x0B, /* Marker Type - Date                 */
    AMF0_LONG_STRING_MARKER     = 0x0C, /* Marker Type - Long string          */
    AMF0_UNSUPPORTED_MARKER     = 0x0D, /* Marker Type - Unsupported          */
    AMF0_RECORD_SET_MARKER      = 0x0E, /* Marker Type - Record set           */
    AMF0_XML_DOC_MARKER         = 0x0F, /* Marker Type - XML document         */
    AMF0_TYPED_OBJECT_MARKER    = 0x10, /* Marker Type - Typed object         */
    AMF0_AVMPLUS_OBJECT_MARKER  = 0x11, /* Marker Type - AVM plus object      */
    AMF0_INVALID_MARKER         = 0xff  /* Marker Type - Invalid              */

} AMF0_DATA_TYPE;

/*********************
 * Structures
 *********************/

typedef struct _EventBuff
{
    UINT16                  EventType;  /* User control message event type    */
    INT32                   StreamId;   /* Stream Identifier                  */

}EventBuff;

typedef struct _AmfStrObj
{
    UINT8                   *StrData;   /* Pointer to string object data      */
    UINT16                  StrLen;     /* Length of string object data       */

} AmfStrObj;

typedef struct _AmfObj
{
    INT32                   ObjCnt;         /* Total obj properties in list   */
    struct _AmfObjProp      *ObjPropList;   /* Object property list           */
    struct _AmfObjProp      *ObjPropListEnd;/* Pointer to end of object 	 */

} AmfObj;

typedef union _AmfObjVal
{
    BOOL                    BoolObj;    /* Object Value - Boolean type value  */
    DOUBLE                  NumObj;     /* Object Value - Number type value   */
    AmfStrObj               StrObj;     /* Object Value - String type value   */
    struct _AmfObj          Obj;        /* Object Value - Object type value   */

} AmfObjVal;

typedef struct _AmfObjProp
{
    AmfStrObj               ObjName;    /* Name of object property            */
    AMF0_DATA_TYPE          ObjType;    /* Type of object property            */
    AmfObjVal               ObjVal;     /* Value of object property           */
    INT16                   UTCOff;     /* UTC offset value                   */

    struct _AmfObjProp      *NextProp;  /* Pointer to next object property    */
    struct _AmfObjProp      *PrevProp;  /* Pointer to prev object property    */

} AmfObjProp;

/************************
 * Function declarations
 ************************/

 /////////////////////////////////// DECODE ////////////////////////////////////

/* Function reads sequential buffer containing Amf encoded object and fills
   the AmfObj type structure                                                  */
INT32 DecodeAmfBuffer (SHANDLE serverHandle, AmfObj *amfObj, UINT8 *payloadBuff,
                       INT32 noOfBytesRemain, BOOL bDecodeName);

/* Function reads sequential bufffer containing Amf encoded property and
   fills AmfObjProp type structure                                            */
INT32 ReadAmfProp (SHANDLE serverHandle, AmfObjProp *amfObjProperty,
                   UINT8 *encodedBuff, INT32 noOfBytesRemain, BOOL bDecodeName);

/* Add property in the object                                                 */
VOID AddAmfProp (SHANDLE serverHandle, AmfObj *amfObj, AmfObjProp *propObj);

/* Function reads amf encoded number property and fills it in double type
   variable                                                                   */
DOUBLE ReadAmfNumber (SHANDLE serverHandle, UINT8 *encodedBuff);

/* Function reads amf encoded boolean property and fills it in boolean
   type variable                                                              */
inline BOOL ReadAmfBoolean(SHANDLE serverHandle, UINT8 *encodedBuff);

/* Function reads sequential buffer containing Amf encoded string property
   and fills AmfStrObj type structure                                         */
inline VOID ReadAmfString (SHANDLE serverHandle, UINT8 *encodedBuff,
                           AmfStrObj *strObj);

inline INT32 ReadAmfStrictArray (SHANDLE serverHandle, AmfObj *amfObject, UINT8 *encodedBuff,
                                 INT32 noOfBytesRemain);

/* Function gets AMF object property from object property list                */
AmfObjProp * GetAmfProp (SHANDLE serverHandle, AmfObj *amfObj, AmfStrObj *name, INT32 index);

/* Function gets string value of AMF object property                          */
inline VOID GetAmfPropString (SHANDLE serverHandle, AmfObjProp *amfProp, AmfStrObj *name);

/* Function gets number value of AMF object property                          */
inline DOUBLE GetAmfPropNumber (SHANDLE serverHandle, AmfObjProp *amfProp);

/* Function gets name of AMF object property                                  */
inline VOID GetAmfPropName (SHANDLE serverHandle, AmfObjProp *amfProp, AmfStrObj *name);

/* Function gets type of AMF object property                                  */
inline AMF0_DATA_TYPE GetAmfPropType (SHANDLE serverHandle, AmfObjProp *amfProp);

/* Function gets boolean value of AMF object property                         */
inline BOOL GetAmfPropBoolean (SHANDLE serverHandle, AmfObjProp *amfProp);

/* Function gets object value of AMF object property                          */
inline VOID GetAmfObject (SHANDLE serverHandle, AmfObjProp *amfProp, AmfObj *amfObj);

/////////////////////////////////// ENCODE  ////////////////////////////////////

/* Function to calculates size of AMF encoded buffer                          */
INT32 GetAmfEncBuffSize (SHANDLE serverhandle, AmfObj *sendAmfObj);

/* Function to generate AMF encoded payload data                              */
UINT8* EncodeAmfBuffer (SHANDLE serverhandle, INT32 payloadSize, AmfObj *sendAmfObj);

/* Function to write AMF string in AMF object property                        */
INT32 WriteAmfString (SHANDLE serverhandle, AmfObj *amfObj, UINT8 *nameString, UINT8 *valueString);

/* Function to write AMF number in AMF object property                        */
INT32 WriteAmfNumber (SHANDLE serverhandle, AmfObj *amfObj, UINT8 *nameString, DOUBLE amfNum);

/* Function to write AMF object in AMF object property                        */
INT32 WriteAmfObject (SHANDLE serverhandle, AmfObj *amfObj, AmfObj *encAmfObj);

/* Function to write null in AMF object property                              */
INT32 WriteAmfNull (SHANDLE serverhandle, AmfObj *amfObj);

/* Function to write AMF boolean in AMF object property                       */
INT32 WriteAmfBoolean (SHANDLE serverhandle, AmfObj *amfObj, UINT8 *nameString, BOOL boolFlag);

/* Function to write AMF array in AMF object property                         */
INT32 WriteAmfArray (SHANDLE serverhandle, AmfObj *amfObj, AmfObj *ecmaAmfObj);

/* Function to write AMF double in AMF object property                        */
VOID WriteDouble (SHANDLE serverhandle, DOUBLE amfNum, UINT8 *buffer);

/* Function to free memory allocated for AMF object structure                 */
INT32 FreeAmfObject (SHANDLE serverhandle, AmfObj *tmpAmfObj, UINT8 stringFreeFlag);

#endif  /* _INCLUDE_SERVERAMF_H */

/*******************************************************************************
 * Copyright (c) 2010 eInfochips - All rights reserved.
 *
 * This software is authored by eInfochips and is eInfochips' intellectual
 * property, including the copyrights in all countries in the world.
 * This software is provided under a license to use only with all other rights,
 * including ownership rights, being retained by eInfochips.
 *
 * File may not be distributed, copied, or reproduced in any manner,
 * electronic or otherwise, without the written consent of eInfochips.
 ******************************************************************************/
