/*******************************************************************************
*
* The content of this file or document is CONFIDENTIAL and PROPRIETARY
* to Mobilygen Corporation.  It is subject to the terms of a
* License Agreement between Licensee and Mobilygen Corporation.
* restricting among other things, the use, reproduction, distribution
* and transfer.  Each of the embodiments, including this information and
* any derivative work shall retain this copyright notice.
*
* Copyright 2010 Mobilygen Corporation
* All rights reserved.
*
* QuArc is a registered trademark of Mobilygen Corporation.
*
*******************************************************************************/

#ifndef __RTMPWRITER_H
#define __RTMPWRITER_H

/// \defgroup wscd_rtmpgroup RTMP Bitstream Writer
/// \{

////////////////////////////////////////////////////////////////////////////////
///
/// \page rtmpwriter RTMP Bitstream Writer
/// \latexonly
/// \chapter{RTMP Bitstream Writer}
/// \endlatexonly
///
/// The RTMP writer is a bitstream writer that interfaces to the SDK's
/// RT server stack. The server is capable of streaming payloads of the
/// codec types as listed below,
///
///   Video Codecs
///
///     1) H.264
///
///   Audio Codec
///
///     3) AAC
///
///

#include "writestreamer.h"
#include "ServerCommon.h"

#include <net/if.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>


enum {
    RTMP_STACK_CAPABILITIES_PLATFORM_API = 1,
};

#define RTMP_STACK_CAPABILITIES     RTMP_STACK_CAPABILITIES_PLATFORM_API

/// \brief Default session name
#define DEFAULT_SESSION_NAME        "livestream"

/// \brief Default network interface
#define DEFAULT_INTERFACE           "ANY"

/// \brief Default RTMP port
#define DEFAULT_RTMP_PORT           1935

/// \brief Default RTMPT port
#define DEFAULT_RTMPT_PORT          8080

/// \brief Small buffer size
#define SML_BUFF_SIZE               16

/// \brief Medium buffer size
#define MDM_BUFF_SIZE               128

/// \brief SPS Nal type
#define NALU_SPS                    7

/// \brief PPS Nal type
#define NALU_PPS                    8

/// \brief Max SPS header size
#define SPS_DATA_SIZE               1000

/// \brief AAC config size
#define AAC_CONFIG_SIZE             2

/// \brief Qbox header size
#define QBOX_HEADER_SIZE            28

/// \brief Enum for audio video flag
typedef AV_FLAG RTMP_AV_FLAG;

/// \brief rtmpwriter structure stores all informations related to the created
/// bitstream writer using rtmpWriterCreate() function
typedef struct
{
    /// \brief RTMP server handle
    SHANDLE             ServerHandle;
    /// \brief RTMP session handle
    SHANDLE             SessionHandle;
    /// \brief Network interface to use
    UINT8               NetworkInterface [SML_BUFF_SIZE];
    /// \brief Session name
    UINT8               SessionName [MDM_BUFF_SIZE];
    /// \brief RTMP port number
    UINT16              RtmpPortNo;
    /// \brief RTMPT port number
    UINT16              RtmptPortNo;
    /// \brief Audio-Video flag
    RTMP_AV_FLAG        AvFlag;
    /// \brief Chunk Size
    INT32               PreferedChunkSize;
    /// \brief Video bitrate
    INT32               VideoDataRate;
    /// \brief Audio bitrate
    INT32               AudioDataRate;
    /// \brief Flag to indicate audio information added
    UINT8               AdInfoAdded;
    /// \brief Flag to indicate audio specific config value found
    UINT8               AacConfigFound;
    /// \brief AAC config data
    UINT8               AacConfigData [AAC_CONFIG_SIZE];
    /// \brief Flag to indicate video information added
    UINT8               VdInfoAdded;
    /// \brief SPS header found flag
    UINT8               SpsFoundFlag;
    /// \brief SPS data size
    INT32               SpsDataSize;
    /// \brief SPS header
    UINT8               SpsData [SPS_DATA_SIZE];
    /// \brief PPS data size
    INT32               PpsDataSize;
    /// \brief PPS header
    UINT8               PpsData [SPS_DATA_SIZE];
    /// \brief SPS PPS Data Size
    INT32               SpsPpsDataSize;
    /// \brief SPS PPS Data
    UINT8               SpsPpsData[SPS_DATA_SIZE+SPS_DATA_SIZE];
    /// \brief Audio info structure
    RtmpAudioInfo       AdInfo;
    /// \brief Video info structure
    RtmpVideoInfo       VdInfo;
    /// \brief Fractional framerate flag
    INT32               fr1001;
    /// \brief Framerate
    INT32               fr;
    /// \brief Key frame frequency
    INT32               kfi;
    /// \brief IP DSCP value
    UINT8               dscp;

    /// \brief AVCENC Handle
    UINT32              avcencHandle;
    /// \brief Clock rate
    INT32               tick_120;

    RtmpStatus          status;
    RtmpLog             log;

    /// \brief Bandwidth monitor
    UINT8   bwmEnable;
    INT32   bwm_size;

    /// \brief Sender options
    UINT8   senderType;
    INT32   sender_audioBufferSize;
    INT32   sender_videoBufferSize;

    /// \brief Debug settings
    UINT8   debugEnable;
    UINT8   dbg_burnin;
    UINT8   dbg_file;
    UINT8   dbg_path[MDM_BUFF_SIZE];
    INT32   dbg_interval_frames;

    /// \brief Adaptive bitrate settings
    INT32   adaptiveBitrate;
    INT32   adapt_max; // bytes (absolute max)
    INT32   adapt_min; // bytes (absolute min)
    INT32   adapt_settleTime; // ms
    INT32   adapt_recoverThreshold; // ms
    INT32   adapt_recoverStepPercent; // %
    INT32   adapt_recoverMinStep; // bytes
    INT32   adapt_recoverInterval; // sec
    INT32   adapt_adjustThreshold; // ms
    INT32   adapt_adjustPercent; // % of measured bandwidth
    INT32   adapt_triggerThreshold; // ms
    INT32   adapt_decreasePercent; // % of measured bandwidth
    INT32   adapt_decreaseInterval; // sec
    INT32   adapt_margin; // % jitter margin (% of limit)

    INT8    pub_args[4096];

    THREAD_HANDLE rtmpConnectThreadId;

} RTMPWRITER_HANDLE;

/// \brief Create an instance of the bitstream writer
/// return Handle to the bitstream writer
BITSTREAMWRITER_HANDLE rtmpWriterCreate          ();

/// \brief Open the bitstream writer
/// \param h Handle to the bitstream writer
/// \param name Comma-separated arguments
///
/// This function is used to open a bitstream writer.  Note that this function
/// is not called directly from the application but instead called from the
/// writestreamer's open function.
int                    rtmpWriterOpen            (BITSTREAMWRITER_HANDLE h,
                                                  const char *name);

/// \brief Close the bitstream writer and its open file
/// \param h Handle to the bitstream writer
/// \return 1 for success, 0 for failure
int                    rtmpWriterClose           (BITSTREAMWRITER_HANDLE h);

/// \brief Send a new block of data to the writer
/// \param h Handle to the bitstream writer
/// \param buf Pointer to the compressed data block (qbox)
/// \param len Length in bytes of the compressed data block
///
/// This function is used to send new data to the writer. The buffer then will
/// be passed to the server stack for sending over the network. Server stack
/// will packetize the buffer in RTMP packets
int                    rtmpWriterNewBlock        (BITSTREAMWRITER_HANDLE h,
                                                    char *buf, int len);

/// \brief Send a new block of data to the writer via zero copy
/// \param h Handle to the bitstream writer
/// \param buf Pointer to the compressed data block (qbox)
/// \param len Length in bytes of the compressed data block
/// \param fd File descriptor for zero copy
/// \param vzero vZero pointer for zero copy
///
/// This function is used to send data to the writer with the intention of
/// using a zero copy (sendfile) operation.
int                    rtmpWriterNewBlockUMA     (BITSTREAMWRITER_HANDLE h,
                                                char *buf, int len, int fd,
                                                void *vzero);

/// \brief Unified function for both zero copy and two copy mode, both
/// rtmpWriterNewBlockUMA() and rtmpWriterNewBlock() will call this function
/// for further processing with some more arguments.
///
/// \param handle Handle to the bitstream writer
/// \param buf Pointer to the compressed data block (qbox)
/// \param len Length in bytes of the compressed data block
/// \param uma Flag to indicate zero copy or two copy to be used
/// \param fd File descriptor for zero copy
/// \param vzero vZero pointer for zero copy
int                    rtmpWriterNewBlockUnified (BITSTREAMWRITER_HANDLE handle,
                                        char *buf, int len, int uma,
                                        int fd, void *vzero);

int                    rtmpWriterIOCTL (BITSTREAMWRITER_HANDLE handle, unsigned int cmd, void *arg);

extern int publishRtmpConnect(RtmpServer *rtmpServer, PublishServer *publish, char *session, char *ret);
extern int publishRtmpConnectThread(void *arg);
extern int publishRtmpClose(RtmpServer *rtmpServer, PublishServer *publish, char *session, char *ret);
extern INT32 publishGetState(RtmpServer *rtmpServer, char *session, char *ret);
extern INT32 parseRtmpUrl(INT8 *url, INT8 *ip, INT32 *port, INT8 *file);


/// \}
#endif //#ifndef __RTMPWRITER_H
