/*******************************************************************************
*
* The content of this file or document is CONFIDENTIAL and PROPRIETARY
* to Mobilygen Corporation.  It is subject to the terms of a
* License Agreement between Licensee and Mobilygen Corporation.
* restricting among other things, the use, reproduction, distribution
* and transfer.  Each of the embodiments, including this information and
* any derivative work shall retain this copyright notice.
*
* Copyright 2007 Mobilygen Corporation
* All rights reserved.
*
* QuArc is a registered trademark of Mobilygen Corporation.
*
*******************************************************************************/

#ifndef __RTPWRITER_H
#define __RTPWRITER_H

/// \defgroup wscd_rtpgroup RTP/RTSP Bitstream Writer
/// \{

////////////////////////////////////////////////////////////////////////////////
///
/// \page rtpwriter RTP/RTSP Bitstream Writer
/// \latexonly
/// \chapter{RTP/RTSP Bitstream Writer}
/// \endlatexonly
///
/// The RTP writer is a bitstream writer that interfaces to the SDK's
/// RTP/RTSP server stack. The server is capable of streaming payloads of the
/// codec types as listed below over RTP,
///
///   Video Codecs
///
///     1) H.264
///
///     2) MJPEG
///
///   Audio Codec
///
///     3) AAC
///
///   Speech Codecs
///
///     4) G.711 (A-Law and U-Law)
///
///     5) G.726
///
///     6) PCM (L16)
///
///      Transport Stream
///
///        7) MPEG-2 Transport Stream
///
/// RTP can be over TCP or UDP depending on the client and RTSP is over TCP only.
/// It follows the following RFCs:
///
/// RFC 3984: RTP Payload Format for H.264 Video
///
/// RFC 3550: RTP A Transport for Real-Time Applications
///
/// RFC 2326: RTSP Real Time Transport Protocol
///
/// RFC 2435: RTP Payload Format for MJPEG Video
///
/// RTCP session can take place along with RTP on the same transport mechanism.
/// It follows the following RFC:
///
/// RFC 3550: RTP A Transport for Real-Time Applications
///
/// The stack is capable of running one server with multiple sessions on
/// a single port, or running multiple servers on multiple ports. Each
/// bitstream writer specifies the port number in its filename argument.
/// If the port number is reused then a new session is added to an
/// existing server, if the port number is previously unused then a new
/// server on that port is created.
///
/// The server is also capable of tunneling its RTSP requests through HTTP, the
/// http port can be given via filename arguments. If the http port is not given
/// session will be created on port 8080.
///
/// The created session can be in one of the two modes, unicast or
/// multicast. If Multicast IP and Port are specified then the session
/// will be created in Multicast mode and will stream the data to that
/// particular multicast IP, If Multicast IP is specified as the string
/// 'NULL' then session will be created in unicast mode.
///
/// For multicast mode, all the routers connected between the server and client
/// path must have multicast enabled in it. You can use any router which
/// supports multicasting protocols (i.e. IGMP and PIM). Alternatively the
/// server and the client can be connected in same L2 switch or hub.
/// If the multicast rtsp client is running on EVP board, all-multi option
/// must be enabled on its network interface.
/// e.g. ifconfig <interface> allmulti
///
/// The stack is also capable to stream RTP packets to the client without
/// establishing RTSP connection which are called Third Party Clients,
/// third party client IP and Ports can be specified via  filename argument.
///
/// The server responds to RTSP requests using URLs of the following
/// form
///
/// rtsp://server:port/session
///
/// Note that if the port is set to well-known default of 554 then the port
/// number is assumed and need not be specified by the client.  However,
/// the server can only be created on this port if the user is running as
/// root.
///
/// The server also supports devices which have multiple network
/// interfaces.  The IP address of the server can be specified directly,
/// or it can be obtained by querying the interface (which itself can be
/// specified in the arguments).  If the server IP is specified as
/// the string 'NULL' then it queries the interface 'eth0' for its address.
///
/// RTCP Support:
///
/// The server also supports the RTCP session alongwith the RTP, with the same
/// transport mechanism as used by RTP. By default, RTCP session is disabled.
/// The server is capable of sending the Sender Report(SR) Packets and BYE
/// Packets to all the connected clients It sends the SR packets periodically at
/// regular interval, while sends the BYE packet only at the end of the session.
///
/// The Sender Report(SR) Packets are used to send the statistics of the Sender
/// of RTP packets. Sender Report contains the statistics which are used by
/// the client for the control of the RTP stream.
///
/// It contains the NTP and RTP timestamps which can be used by the client
/// to achieve audio-video sync. The NTP timestamp represents the wall-clock
/// time, but in the NTP format. The RTP timestamp is calculated in the same
/// way as calculated in the RTP packets, using the same time as reference, as
/// used to calculate the NTP timestamp. At the client-side, it takes the NTP
/// timestamp, which gives the wall-clock and takes the RTP timestamp which is
/// corresponding to that wall-clock time. Thus the client collects this
/// information for each stream from the RTCP packets of each stream. Using this
/// information, it can calculate the presentation time of each stream, which
/// will ultimately lead to audio-video sync.
///
/// For e.g. the VLC player client uses the RTCP packets to achieve the
/// audio-video sync. When it receives SR packet, it stores the NTP and
/// RTP timestamp and based on that calculates presentation time as shown below:
///
/// SR_NTPTS = NTP timestamp received in SR packet
/// SR_RTPTS = RTP timestamp received in SR packet
/// CP_RTPTS = RTP timestamp of currently received RTP packet
///
/// Now to calculate the presentation time of any frame first it calculates the
/// diff value of RTP timestamp.
///
/// Diff = CP_RTPTS - SR_RTPTS
/// Diff is converted to presentation time unit.
/// Presentation time = SR_NTPTS(represented in presentation time unit) + Diff
///
/// Similarly, the SR packets contains other statistics which the client can use
/// to control (or analyze) the RTP. The Server also sends RTCP BYE packet which
/// is to inform that it is closing the session.
///
/// Configuration Options:
///
/// The filename argument to the writer is interpreted as a series
/// of comma-separated arguments.  The arguments are as follows.
///
/// - inf : The interface to query for IP address (eth0 by default if not specified)
/// - sip : The server IP address, or NULL for the specified interface's address
/// - prt : The sever port number
/// - ses : The session name. The session is appended to the URL of the form rtsp://sip:prt/ses
/// - vid : Video codec. It can be h264, MJPEG at this time
/// - aud : Audio codec. It can be AAC, G.711 (A-Law and U-Law) or G.726 at this time
/// - msz : MTU size in case of RTP over UDP. Typically 1472 for ethernet
/// - ums : MTU size in case of RTP over UDP. Typically 1472 for ethernet
/// - tms : MTU size in case of RTP over TCP. Typically 8192 for ethernet
/// - mip : Multicast IP address. For the multi-cast address, you can use any
///         class D IP addresses (224.0.0.0 to 239.255.255.255), but out of
///         this range some are reserved for special uses which we should not
///         use, so it is better to use unassigned range (225.0.0.0 to 238.255.255.255).
///         For more information on multicast address ranges you can refer following
///         link:
///         http://www.iana.org/assignments/multicast-addresses/
/// - mptv: Multicast Video Port
/// - mpta: Multicast Audio Port
/// - mcli: Max allowed client connection
/// - cli : Third Party client IP
/// - cpt : Third Party client Port.
/// - mod : Server mode.  Single mode or Normal mode (default is Normal mode).
/// - htp : HTTP port number
/// - aim : Flag to indicate mechanism (private SEI or header extension) to use for Analytics streaming
/// - rtcp: Flag to indicate whether RTCP session is to be started
/// - sri : RTCP Sender Report interval in milliseconds
/// - mp2t: Flag to indicate Transport Stream
/// - mpts: Multicast Transport Stream Port

#include "writestreamer.h"

#include "RtpRtsp.h"

#include <net/if.h>

#include <sys/ioctl.h>

#include <arpa/inet.h>

/// \brief Clock rate in case of H.264 and MJPEG streams
#define CODEC_CLOCK_RATE    90000

/// \brief Clock rate in case of G.711 streams
#define G711_CLOCK_RATE     8000

/// \brief Max name string length
#define MAX_NAME_LENGTH     32

/// \brief SPS header value
#define NALU_SPS            7

/// \brief PPS header value
#define NALU_PPS            8

/// \brief Default UDP MTU size
#define UDP_MTU_SIZE        1450

/// \brief Default TCP MTU size
#define TCP_MTU_SIZE        8192

/// \brief Qbox header size
#define QBOX_HEADER_SIZE    28

/// \brief  SPS data size
#define SPS_DATA_SIZE       1000

/// \brief Default network interface
#define INTERFACE           "eth0"

/// \brief Default HTTP port
#define SERVER_HTTP_PORT    8080

/// \brief SEI Buffer Size
#define SEI_BUFF_SIZE       128

/// \brief RTP Header extension type
/// 0x10 - Send MetaData (Object Tracking/ Motion Detection)
#define RTP_HE_TYPE         0x10

/// \brief IOCTL command ID for rtpWriterNewBlock()
#define FS_NEWBLOCK         (1 << 1)

/// \brief IOCTL command ID for rtpWriterNewBlockUMA()
#define FS_NEWBLOCK_UMA        (1 << 2)

#define Q_WS_IOCTL_RTSPWRITER_CONNECT 10
#define Q_WS_IOCTL_RTSPWRITER_DISCONNECT 11

/// \brief Link list of third clients
struct TP_CLIENT
{
    /// \brief IP address of third party client
    unsigned char clientIP[MAX_NAME_LENGTH];
    /// \brief Port number of third party client
    int           clientPort;
    /// \brief TTL for multicast
    int           clientTTL;
    /// \brief Pointer to the next client
    struct TP_CLIENT *nextClient;
};

typedef struct TP_CLIENT TP_CLIENT;

#define RTP_SENDER_THREAD
#ifdef RTP_SENDER_THREAD

#define RTP_SENDER_QUEUE_LENGTH     512
#define RTP_SENDER_QUEUE_CLEAR      90
#define RTP_SENDER_SEND_COUNT       90
#define RTP_SENDER_QUEUE_HIGH_FLUSH (RTP_SENDER_QUEUE_LENGTH-RTP_SENDER_QUEUE_CLEAR)
#define RTP_SENDER_HEADER_CHECK     QBOX_HEADER_SIZE

typedef struct
{
    BITSTREAMWRITER_HANDLE handle;

    int valid;
    int tspacket;
    int dataType;

    char header[RTP_SENDER_HEADER_CHECK];

    char *buf;
    int len;
    int uma;
    int fd;
    void *vzero;
    int qBox;
    void *conn_node;
    int vidCodecAdded;
    int audCodecAdded;
    SupportedCodecs  VideoCodecType;
    SupportedCodecs  AudioCodecType;
    SupportedCodecs  TSCodecType;
    unsigned int ntp_time;

} RTP_SENDER_DATA;

typedef struct
{
    int size;
    int high;

    // Queue pointers
    int head;
    int tail;
    int max;
    int count;

    // Counters
    int send_count;
    int high_water;
    int stalled;

    int queue_reset_mode;
    int queue_reset_time;

    pthread_mutex_t lock;
    pthread_cond_t  ready;
    RTP_SENDER_DATA *data;

} RTP_SENDER_QUEUE;

#endif

/// \brief rtpwriter structure stores all informations related to the created
/// bitstream writer using rtpWriterCreate() function
typedef struct
{
    /// \brief HTTP port number
    int  httpPortNo;
    /// \brief RTSP port number
    int  rtspServerPort;
    /// \brief Flag to discard the frames bigger than specified by user
    int  rtspSingleModeFlag;
    /// \brief Flag to indicate sps found or not
    int  spsHeaderFound;
    /// \brief User given MTU size for UDP
    int  rtpPktUdpMtuSize;
    /// \brief User given MTU size for TCP
    int  rtpPktTcpMtuSize;
    /// \brief RTSP server handle
    int  rtspServerHandle;
    /// \brief RTSP session handle
    int  rtspSessionHandle;
    /// \brief User given Username for Client authentication
    char *rtspUserName;
    /// \brief User given Password for Client authentication
    char *rtspPassword;
    /// \brief User given RTSP server IP
    char rtspServerIP[MAX_NAME_LENGTH];
    /// \brief User given session name
    char rtspSessionName[MAX_NAME_LENGTH];
    /// \brief Flag to indicate whether RTCP session enable or not
    int  rtcpFlag;
    /// \brief Interval between two RTCP SR packets
    int  rtcpSRInterval;
    /// \brief RTSP Session-timeout interval for the session
    int  rtspSessionTimeout;
    /// \brief Flag to indicate Transport Stream
    int     rtspTS;
    /// \brief User given video codec Type
    char rtspVideoCodec[MAX_NAME_LENGTH];
    /// \brief User given audio codec Type
    char rtspAudioCodec[MAX_NAME_LENGTH];
    /// \brief User given Network interface
    char nwInterface [MAX_NAME_LENGTH];
    /// \brief Flag to indicate wether audio stream added or not
    int  addAudioStream;
    /// \brief Flag to indicate wether video stream added or not
    int  addVideoStream;
    /// \brief Flag to indicate wether transport stream added or not
    int  addTransportStream;
    /// \brief Size of sps extracted from the encoded buffer
    int  spsDataSize;
    /// \brief SPS extracted from the encoded buffer
    char spsData [SPS_DATA_SIZE];
    /// \brief Size of pps extracted from the encoded buffer
    int  ppsDataSize;
    /// \brief PPS extracted from the encoded buffer
    char ppsData [SPS_DATA_SIZE];
    /// \brief SDP data size
    int  sdpDataSize;
    /// \brief Buffer required for SDP generation
    char sdpData[SPS_DATA_SIZE+SPS_DATA_SIZE];
    /// \brief Total number of connected third party clients
    int  noOfTpClient;
    /// \brief List of third party clients
    TP_CLIENT *tpClientList;
    /// \brief Filestreamer callback function
    FS_EVENT_CALLBACK FS_Callback;
    /// \brief File end time in second
    int fileEndTime;
    /// \brief Multicast video Port
    int multicastPortVideo;
    /// \brief Multicast audio Port
    int multicastPortAudio;
    /// \brief Multicast ts Port
    int multicastPortTS;
    /// \brief Multicast IP
    char multicastIP[MAX_NAME_LENGTH];
    /// \brief Multicast TTL
    char multicastTTL;
    /// \brief Meta data buffer
    char metaDataBuffer[WRSTREAMER_METADATA_MAX_SIZE];
    /// \brief Size of meta data
    int  metaDataSize;
    /// \brief Flag to indicate mechanism to use for analytics streaming
    /// 0 - Don't send any metadata
    /// 1 - Analytics data in metaData
    /// 2 - Analytics data in SEI
    int  metaDataFlag;
    /// \brief Private SEI buffer
    char privateSEI[SEI_BUFF_SIZE];
    /// \brief Size of private SEI
    int privSeiSize;
    /// \brief Client's socket send buffer limit in terms of percentage
    int SendBuffLimitInPercent;
    /// \Maximum client allowed
    int MaxClient;

    unsigned int streamSSRC;

    int cur_bitrate;
    int tick_120;
    unsigned int queue_time;
    struct timeval last_pict_adj_time;
    int fraction_packets_lost;
    int fraction_packets_lost_prev;
    int roundtrip_delay;
    int bitrate_lim;
    int bitrate_prev_cong;
    // Startup parameters:
    int adapt_br;
    int disp_br;
    int max_br;
    int up_speed_br;
    int down_speed_br;
    int sens_br;

#ifdef RTP_SENDER_THREAD
    int                 rtp_sender_enable;
    pthread_t           rtp_sender_thread_id;
    RTP_SENDER_QUEUE    rtp_sender_queue;
#endif

} RTPWRITER_HANDLE;

/// \brief IOCTL argument for filestreamer
typedef struct
{
    /// \brief Pointer to the compressed data block (qbox)
    char *buffer;
    /// \brief Length in bytes of the compressed data block
    int length;
    /// \brief Client to which RTP data to be sent
    void *clientid;
    /// \brief fd File descriptor for zero copy
    int fd;
    /// \brief vzero vZero pointer for zero copy
    void *vzero;
} RTPWRITER_IOCTL_ARG;

/// \brief Create an instance of the bitstream writer
/// return Handle to the bitstream writer
BITSTREAMWRITER_HANDLE rtpWriterCreate          ();

/// \brief Open the bitstream writer
/// \param h Handle to the bitstream writer
/// \param name Comma-separated arguments
///
/// This function is used to open a bitstream writer.  Note that this function
/// is not called directly from the application but instead called from the
/// writestreamer's open function.
int                    rtpWriterOpen            (BITSTREAMWRITER_HANDLE h, char *name);

/// \brief Close the bitstream writer and its open file
/// \param h Handle to the bitstream writer
/// \return 1 for success, 0 for failure
int                    rtpWriterClose           (BITSTREAMWRITER_HANDLE h);

/// \brief Send a new block of data to the writer
/// \param h Handle to the bitstream writer
/// \param buf Pointer to the compressed data block (qbox)
/// \param len Length in bytes of the compressed data block
///
/// This function is used to send new data to the writer. The buffer then will be
/// passed to the server stack for sending over the network. Server stack will
/// packetize the buffer in RTP packets if needed.
int                    rtpWriterNewBlock        (BITSTREAMWRITER_HANDLE h,
                                                char *buf, int len);
/// \brief Call an IOCTL on the writer
/// \param h Handle to the writer
/// \param cmd Command value of the IOCTL
/// \param arg Void pointer representing an argument to the command
///
/// The rtpwriter does not support any IOCTL calls.
int                    rtpWriterIOCTL           (BITSTREAMWRITER_HANDLE h,
                                                unsigned int cmd, void *arg);

/// \brief Display verbose information about the writer
/// \param h Handle to the bitstream writer
/// \return 1 for success, 0 for failure
int                    rtpWriterShow (BITSTREAMWRITER_HANDLE h);

/// \brief This operation is not supported in RTP writer
int                    rtpWriterPwrite          (BITSTREAMWRITER_HANDLE h,
                                                char *buf, int count, int offset);

/// \brief Send a new block of data to the writer via zero copy
/// \param h Handle to the bitstream writer
/// \param buf Pointer to the compressed data block (qbox)
/// \param len Length in bytes of the compressed data block
/// \param fd File descriptor for zero copy
/// \param vzero vZero pointer for zero copy
///
/// This function is used to send data to the writer with the intention of
/// using a zero copy (sendfile) operation.
int                    rtpWriterNewBlockUMA     (BITSTREAMWRITER_HANDLE h,
                                                char *buf, int len, int fd,
                                                void *vzero);

/// \brief Unified function for both zero copy and two copy mode, both
/// rtpWriterNewBlockUMA() and rtpWriterNewBlock() will call this function
/// for further processing with some more arguments.
///
/// \param handle Handle to the bitstream writer
/// \param buf Pointer to the compressed data block (qbox)
/// \param len Length in bytes of the compressed data block
/// \param uma Flag to indicate zero copy or two copy to be used
/// \param fd File descriptor for zero copy
/// \param vzero vZero pointer for zero copy
/// \param conn_node connection node
int                    rtpWriterNewBlockUnified    (BITSTREAMWRITER_HANDLE handle,
                                                char *buf, int len, int uma,
                                                int fd, void *vzero, void *conn_node);

/// \brief Configure writer parameters
/// \param handle Handle to the bitstream writer
/// \param param Parameter index to configure
/// \param value Parameter value to set
/// \return 1 for success, 0 for failure
int                    rtpWriterConfig          (BITSTREAMWRITER_HANDLE handle,
                                                int param, int value);
/// \brief Used to send new metadata to the writer
/// \param handle Handle to the bitstream writer
/// \param buf buffer data
/// \param length buffer size
/// \param type Type of stream
/// \param streamID ID of Stream
/// \return 1 for success, 0 for failure
int                    rtpWriterNewMetadata      (BITSTREAMWRITER_HANDLE handle, char *buf,
                                                 int length, int type, int streamID);

/// \}
#endif //#ifndef __RTPWRITER_H
