/*
 * Copyright, 2009, Teradek
 * 
 * All rights reserved.
 * Contact Teradek for licensing.
 */



#ifndef _IPC_H_
#define _IPC_H_

#include <stdio.h>

#ifdef __cplusplus
  extern "C" {
#endif

/** <h3>Teradek Libraries</h3> 
 * \addtogroup libtd
 * \{
 */

/** Path to the named socket used for the IPC.
 * 
 * When an IPC server is started, it creates a UNIX socket that is used by clients to
 * connect to the server.  This is the path where the named socket is located.
 */
#define IPC_SOCK_PATH           "/tmp"

/** Used to delimit a IPC message
 * 
 * This is internally used to delimit the IPC message.  You should not 
 * need this in your application.
 * NOTE: This can not be part of the application layer protocol.
 */
#define IPC_COMMAND_DELIMITER   "@\n"


/** Prototype for function to handle IPC connection
 * 
 * \param in A FILE stream used to read data from the server
 * \param out A FILE stream use to write data to the server
 * \param arg Opaque object passed from server
 * \return Not used
 * 
 * This function is called when a client make an IPC connection to the server.  The server will
 * create the in and out stream and pass it to the callback function.
 * 
 * The callback function needs to handle the IPC protocol.  The server does not handle any
 * protocol processing at all.  It just handles the server and client mechanism.
 */ 
typedef int (*ipc_callback)(FILE *in, FILE *out, void *arg);

/** Start the IPC server
 *
 * \param id A string to identify the socket interface.
 * \param numbacklog Number of clients to allow at one time
 * \param handle_client Function pointer to call when a client connects.
 * \param arg Pointer to an object that will be pass in the client callback function.
 * \return Status of call
 * 
 * This function will spawn a thread that will listen for incoming client connections.  It will
 * act as the IPC server.  When a client connects, it will spawn a thread to handle the connection.
 * It create a bidirectional "stream" between the client and the server.  It then calls the 
 * callback function that the application provided.  The callback will handle the communication 
 * between the client and server using the streams.  The application can send data to and read 
 * back data from the server.  The communication protocol is independent of this function and 
 * is implemented in the application.
 */
int td_ipc_server_init(const char *id, int numbacklog, ipc_callback handle_client, void *arg);

/* single thread only but with proper termination
 */
int td_bond_ipc_server_init(const char *id, int numbacklog, ipc_callback handle_client, void *arg);

/** Stop the IPC server
 * 
 * This function will clean up the IPC server.
 */
int td_ipc_server_exit(void);
int td_bond_ipc_server_exit(void);


/** Open a connection to the IPC server
 * 
 * \param id String used to identify the IPC server
 * \param to FILE stream used to write data to the server
 * \param from FILE stream used to read data from the server
 * \return Returns a file descriptor of the connection.  Used for disconnect function
 * 
 * This function is used  
 */
int td_ipc_client_connect_single(char *id, FILE **to, FILE **from);

/** Open a connection to the IPC server - Single thread
 * 
 * \param id String used to identify the IPC server
 * \param to FILE stream used to write data to the server
 * \param from FILE stream used to read data from the server
 * \return Returns a file descriptor of the connection.  Used for disconnect function
 * 
 * This function is used  
 */
int td_ipc_client_connect(char *id, FILE **to, FILE **from);

/** Close the connection to the IPC server
 * 
 * \param sock File descriptor return by connect function.
 * \param to FILE stream return by the connect function.
 * \param from FILE stream returned by the connect function.
 * 
 * This function cleans up any opened socket or FILE stream created by the connect function.
 */
int td_ipc_client_disconnect(int sock, FILE **to, FILE **from);

/** \} */

#ifdef __cplusplus
  }
#endif

#endif /* _IPC_H_ */
