/*
 * Copyright, 2009, Teradek
 * 
 * All rights reserved.
 * Contact Teradek for licensing.
 */



#ifndef _RPC_H_
#define _RPC_H_

/** <h3>Teradek Libraries</h3> 
 * \addtogroup libtd
 * \{
 */

/** Used to delimit a RPC message
 * 
 * This is internally used to delimit the RPC message.  You should not 
 * need this in your application.
 * NOTE: This can not be part of the application layer protocol.
 */
#define RPC_COMMAND_DELIMITER   "@\n"


/** Prototype for function to handle RPC connection
 * 
 * \param in A FILE stream used to read data from the server
 * \param out A FILE stream use to write data to the server
 * \return Not used
 * 
 * This function is called when a client make an RPC connection to the server.  The server will
 * create the in and out stream and pass it to the callback function.
 * 
 * The callback function needs to handle the RPC protocol.  The server does not handle any
 * protocol processing at all.  It just handles the server and client mechanism.
 */ 
typedef int (*rpc_callback)(FILE *in, FILE *out, void *arg);

/** Start the RPC server
 *
 * \param id A string to identify the socket interface.
 * \param numbacklog Number of clients to allow at one time
 * \param handle_client Function pointer to call when a client connects.
 * \return Status of call
 * 
 * This function will spawn a thread that will listen for incoming client connections.  It will
 * act as the RPC server.  When a client connects, it will spawn a thread to handle the connection.
 * It create a bidirectional "stream" between the client and the server.  It then calls the 
 * callback function that the application provided.  The callback will handle the communication 
 * between the client and server using the streams.  The application can send data to and read 
 * back data from the server.  The communication protocol is independent of this function and 
 * is implemented in the application.
 */
extern int td_rpc_server_init(char *address, int port, int numbacklog, rpc_callback callback_func, void *arg);

/** Open a connection to the RPC server
 * 
 * \param id String used to identify the RPC server
 * \param to FILE stream used to write data to the server
 * \param from FILE stream used to read data from the server
 * \return Returns a file descriptor of the connection.  Used for disconnect function
 * 
 * This function is used  
 */
extern int td_rpc_client_connect(char *address, char *port, FILE **to, FILE **from);

/** Close the connection to the RPC server
 * 
 * \param sock File descriptor return by connect function.
 * \param to FILE stream return by the connect function.
 * \param from FILE stream returned by the connect function.
 * 
 * This function cleans up any opened socket or FILE stream created by the connect function.
 */
extern int td_rpc_client_disconnect(int sock, FILE **to, FILE **from);

/** \} */

#endif /* _RPC_H_ */
