/*******************************************************************************
* 
* Teradek(C) 2009
* Transport stream parsing routines
* 
* Questions: ilya.issenin@teradek.com
* 
*******************************************************************************/

#ifndef TSPARSE_H
#define TSPARSE_H

#include <stdint.h>
#include "inout.h"

#define STREAM_TYPE_VIDEO_H264 0x1B
#define STREAM_TYPE_VIDEO_MPEG2 0x02
#define STREAM_TYPE_AUDIO_AAC 0x0F
#define STREAM_TYPE_AUDIO_MP2 0x04		// MPEG2 audio  
#define STREAM_TYPE_AUDIO_MPEG1 0x03	// MPEG1 audio
#define STREAM_TYPE_AUDIO_AC3 0x81	// Dolby-AC3 audio
#define STREAM_TYPE_AUX_DATA 0x06
#define STREAM_TYPE_NONE 0xFF
	
#define TS_PACKET_SIZE 188
#define AUD_BUF_SIZE 300	// about 2.5s of audio, ~60KB

#define MAX_VIDEO_WAIT 5	// when no video packets are coming, it will assume that the last PES was the last PES in current QBOX  
							// after receiving N audio units (1 unit = 1152 samples, ~30 ms)

// values for status update function
#define TS_OK 			0x0001
#define TS_SYNC_LOST 	0x0002
#define TS_CONT_CNT_ERR	0x0004
#define TS_STRUCT_ERR 	0x0008
#define TS_TIMEOUT	 	0x0010
#define TS_NEW_PIDS 	0x0020
#define TS_NEW_ENCR 	0x0040
#define TS_NEW_VMODE	0x0080
#define TS_ERRORS		(TS_SYNC_LOST | TS_CONT_CNT_ERR | TS_STRUCT_ERR | TS_TIMEOUT)

#ifdef __cplusplus
  extern "C" {
#endif

typedef struct {
	int (*inp_func)(uint8_t **buf_ret, int len, inp_info_t *inp_info);	// function to source the data
	inp_info_t *inp_info;												// data necessary for this function to work
	int (*tty_outp_func)(uint8_t *buf, int len, inp_info_t *inp_info);
	int (*status_update_func)(uint32_t status, inp_info_t *inp_info);
	int pes_start;
	int sync_point;
	uint16_t pid;
	uint16_t video_pid;
	uint16_t audio_pid;
	uint16_t data_pid;
	uint8_t video_ccnt;
	uint8_t audio_ccnt;
	uint8_t data_ccnt;
	uint8_t video_encr;
	uint8_t audio_encr;
	uint8_t data_encr;
	uint8_t video_type;
	uint8_t audio_type;
	uint8_t pid_auto;			// 1: get PIDs from the stream; 0 - use predefined video_pid etc. (not implemented)
	uint8_t program_num_auto;	// 1: get program_num from the stream; 0 - use predefined program_num
	char video_type_descr[32];
	char audio_type_descr[32];
	char service_provider_name[32];
	char service_name[32];
	int do_sync;
	int new_pes;
	int sync_error_reported;
	int skip_error;
	uint8_t adapt;
	uint8_t ccnt;
	uint8_t scrambl;
	uint32_t transport_stream_id;
	int pmt_vers;
	int pat_vers;
	int sdt_vers;
	uint32_t program_number;
	uint32_t program_num_auto_ch;
	uint16_t program_map_PID;
	int length;
	int cur_pos;
	uint8_t *buf;
	uint16_t current_PID;
	uint8_t *saved_buf;
	uint8_t new_qbox_packet[TS_PACKET_SIZE];
	// circular buffer to hold audio TS packets (which can be interleaved with video packets in incoming stream)
	uint8_t aud_buf[TS_PACKET_SIZE*AUD_BUF_SIZE];
	int aud_rd_ptr, aud_wr_ptr;
	int aud_units;
	int aud_units_during_video_wait;
	int stop_on_cc_errors;
} ts_parse_stream_t;

// Reads payload data from transport stream, up to len bytes. Returns the number of bytes actually read from the current TS packet
// and pointer to the buffer where data are located.
// If returned value lower than len, call this function again.
// Retuned error values: 0 - no more data (end of file); < 0 - error in the stream (try to sync again)
//
// Limitations: all TS packets for the same PES should not be interleaved with the packets from another PES
// We also ignore all CRCs and continuity counters
int get_ts_data(ts_parse_stream_t *ts_stream, uint8_t **ptr, int len); 
int get_ts_data_preview(ts_parse_stream_t *ts_stream, uint8_t **ptr, int len);	// does not changes the state if not crossing TS packet boundary

// Tells the parser to look for the packet with the PES headers
void sync_ts_to_new_pes(ts_parse_stream_t *ts_stream);

// Tells the parser to look for the packet with the PES headers with the same PID as before
void sync_ts_to_next_pes(ts_parse_stream_t *ts_stream);
	
// Tells the parser to skip till the beginning of a new TS packet
void skip_ts_packet(ts_parse_stream_t *ts_stream);

// Tells the parser to init internal buffers (for audio data) since stream error was encountered
void ts_clean_on_error(ts_parse_stream_t *ts_stream);

// Reset all the PSI info (if no stream is coming for a long time)
void ts_reset_pids(ts_parse_stream_t *tstream);
	
// Init the stream descriptor
void ts_parse_init(ts_parse_stream_t *tstream, int (*inp_func)(uint8_t **buf_ret, int len, inp_info_t *inp_info), inp_info_t *inp_info,
		int (*tty_outp_func)(uint8_t *buf, int len, inp_info_t *inp_info),
		int (*status_update_func)(uint32_t status, inp_info_t *inp_info));

// Extracts 32-bit PTS value from PES in the given TS packet
// Returns 0 if no PTS found
int extract_pts(ts_parse_stream_t *tstream, uint8_t* buf, uint32_t* pts);

// checks if this is the first packet of a frame
int is_frame_start_packet(uint8_t* buf);

// checks if this is the first packet of an I-frame with a given PID
int is_i_frame_start_packet(uint8_t* buf, uint32_t video_pid);

//void dump(uint8_t *buf, int len);

#ifdef __cplusplus
  }
#endif

#endif
