/*******************************************************************************
* 
* Teradek(C) 2009
* Transport stream generation routines
* 
*******************************************************************************/

#ifndef TSGEN_H
#define TSGEN_H

#include <stdint.h>
#include <sys/time.h>
#include <stdarg.h>
#include <syslog.h>

#define TS_ID 23
#define PROG_ID 42
#define PROG_MAP_PID 0x100
#define PSI_PERIOD_MS 400
#define PCR_PERIOD_MS 25
#define PID_INSERT_TIMEOUT 4000
	
#define NIT_PID 0x10
#define SDT_PID 0x11
#define NULL_PID 0x1FFF
	
#define MAX_PID 20		// max number of different PIDs allowed in the stream (not incl. PID 0, 1 and 2)
#define MAX_PROGS 8	// max number of different TS programs in the stream
#define MAX_DVB_INFO_NAME 20

#define AUD_TYPE_MP1 0
#define AUD_TYPE_MP2 1
#define AUD_TYPE_AAC 2
#define AUD_TYPE_NONE 3


typedef struct {
	uint16_t program_number;
	uint16_t program_map_PID;
	uint16_t video_pid;
	uint16_t audio_pid;
	uint8_t audio_type;
	uint8_t orig_prog;
	
	uint8_t service_type;
	char provider_name[MAX_DVB_INFO_NAME+1];
	char service_name[MAX_DVB_INFO_NAME+1];
} prog_info_t;

typedef struct {
	uint8_t *outbuf;	// pointer to the position where next byte is to be written
	uint8_t *(*out_req)(void *handle, int len);	// function to request len bytes of space in output buffer; returns pointer to that space
	void (*commit_write)(void *handle);							// function that is called when requested len bytes is written to the buffer
	void *handle;
	
	// continuity counter stuff
	uint8_t continuity_cnt[MAX_PID+3];
	uint16_t pid_map[MAX_PID];	// maps PID number to the entry of continuity_cnt table
	uint8_t total_cont_pid;
	
	uint16_t transport_stream_id;
	uint8_t stream_pes_pid_list[MAX_PID];	// stream ID from PES packet
	uint8_t stream_type_list[MAX_PID];		// stream type from PMT
	int total_pids;
	uint16_t pcr_PID;
	uint16_t pcr_stream_PID;	// not needed?
	uint16_t aux_pid;
//	int pcr_audio;	// if audio stream is used for syncronization
	uint8_t pmt_vers;
	uint8_t pat_vers;
	uint8_t no_pcr;
	uint8_t no_null_packets;
	
	prog_info_t prog_info[MAX_PROGS];
	int total_prog_num;	// number of programs
	
	int first_pcr; 
	int first_psi;
	int64_t psi_time_prev;
	int64_t psi_time_prev_thread2;
	int64_t pcr_time_prev;
	uint8_t cnt_empty_packet;
	
	int pcr_interval_ms;
	int psi_interval_ms;
	
	uint8_t spi_out;	// when 1, SPI output with FPGA PCR restamping; PSI tables and PCR are generated from another thread (between SPI bursts)
	
	uint8_t tmp_packet_buf[188];
	uint8_t tmp_packet_buf2[188];
	
	int pcr_pid_same_as_av_pid;
	uint8_t pcr_cont_cnt;

	// DVB service info
	uint16_t network_id;
	uint16_t orig_network_id;
	const char *network_name;

	int (*logCallback)(void *handle, int level, const char* fmt, va_list args);
	void* logHandler;
} tstream_t;

void tsgen_set_logging(tstream_t *tstream, int (*logCallback)(void *handle, int level, const char* fmt, va_list args), void* logHandler);

// init stream descriptor
void tstreamInit(tstream_t *tstream);
void tstreamInit_w_tscfg(tstream_t *tstream, uint16_t ts_id, uint16_t ts_prog_num, uint16_t pmt_pid,
    uint8_t ts_ver_num,
    uint16_t pcr_pid,
    uint16_t video_pid,
    uint16_t audio_pid,
    uint16_t aux_pid,
    uint16_t network_id,
    uint16_t orig_network_id,
    uint8_t service_type,
    const char *network_name,
    const char *provider_name,
    const char *service_name,
    uint8_t no_pcr,
    uint8_t spi_out,
    void *handle,
    uint8_t * (*out_req)(void *handle, int len),
    void (*commit_write)(void *handle)
);
void tstreamInit_w_tscfg2(tstream_t *tstream, uint16_t ts_id, 
	prog_info_t *cfg_prog_info,
    uint8_t ts_ver_num,
    uint16_t pcr_pid,
    uint16_t aux_pid,
    uint16_t network_id,
    uint16_t orig_network_id,
    const char *network_name,
    uint8_t no_pcr,
    uint8_t spi_out,
    void *handle,
    uint8_t *(*out_req)(void *handle, int len),
    void (*commit_write)(void *handle)
);

// generate stream for PES data chunk *data of length len
void genTS(tstream_t *tstream, uint8_t *data, int len, int incl_pcr, int random_access, 
	uint32_t pcr_base_33, uint32_t pcr_base, uint32_t pcr_ext, uint16_t ts_pid);

// generate PSI etc. tables or PCR packet (with 0s in place of timestamp) and put them to outbuf location
int genPSI(tstream_t *tstream, uint8_t *outbuf, int pcr);

// generate PCR packet with the given timestamp and put it to outbuf location
int genPCR(tstream_t *tstream, uint8_t *outbuf, uint32_t pcr_base_33, uint32_t pcr_base, uint32_t pcr_ext);

// generate NULL packet
void genNullPacket(tstream_t *tstream, uint8_t *outbuf);

// regenerate continuity counters if PCR PID matches video or audio PID
void restamp_cont_cnt(tstream_t *tstream, uint8_t *buf, int len);

#endif
