/*******************************************************************************
* 
* Teradek(C) 2009
* MPEG2 transport stream reader
* 
*******************************************************************************/ 
#ifndef __UDPTSREADER_H
#define __UDPTSREADER_H

/// \defgroup rsgroup_c UDP Bitstream Reader
/// \{

////////////////////////////////////////////////////////////////////////////////
///
/// \page udpreader UDP Bitstream Reader
/// \latexonly
/// \chapter{UDP Bitstream Reader}
/// \endlatexonly
///
/// The UDP bitstream reader is designed to receive raw QBOX streams via a UDP
/// socket.  Because the UDP server is considered to be "pushing" the data, 
/// the UDP reader is a threaded-object.  The embedded thread receives data
/// and places it into a local buffer while the readstreamer calls and runs
/// GetBlock() function from its (the readstreamer's) context.  Mutexes 
/// protect concurrent access to the buffer's read and write pointers.
///
/// The UDP reader takes a list of comma-separated arguments in its filename
/// as part of the Open() call.  The arguments are as follows.
///
/// - port: The UDP port on which to listen
/// - bufferlen : The size of the internal buffer.  
///
/// The buffer length should be large enough to handle jitter in the AV decoder's
/// receiving of bitstream data.  A reasonable starting point is 1mbyte which 
/// can then be increased or decreased depending on decoder latency.

#include "refdesign.h"
#include "readstreamer.h"
#include "bsreader.h"
	
typedef struct {
    int state;
    void *bufStart;
    void *bufEnd;

    void *readAtGetBlk;
    void *writeAtGetBlk;
    int fullAtGetBlk;

    void *readAtTh;
    void *writeAtTh;
    int fullAtTh;
    
    int lastRecvBytes;
    int totalRecvBytes;

    int lastSentBytes;
    int totalSentBytes;

    int lastDumpedBytes;
    int totalDumpedBytes;

    unsigned int pts;
    unsigned int prevPts;
    
    unsigned int unsyncCount;
    unsigned int bufFullCount;
    unsigned int thCount;
    unsigned int getBlockCount;
} UDPTSREADER_QUERY_DATA;    

/// \brief Create a UDP reader
/// \param srcId Pointer to an array of stream Id's which will be remapped
/// \param codecId Pointer to an array of stream Id's which are sent to the codec
/// \param nStream Number of stream IDs in the arrays
/// \return Handle to a bitstream reader
BITSTREAMREADER_HANDLE udpTsReaderCreate   (int *srcId, int *codecId, int nStreams);

/// \brief Open a bitstream reader.  This function opens the file and starts prefetching
/// bitstream data in anticipation of sending data.
/// \param name Filename to be opened.  The filename is a set of comma separated 
/// arguments.  The caller must specify "port" and "bufferlen".
/// \return 1 for success, 0 for failure
int                    udpTsReaderOpen     (BITSTREAMREADER_HANDLE h, const char *name);

/// \brief Close a bitstream reader's open file.
/// \param h Handle to a bitstream reader
/// \return 1 for success, 0 for failure
int                    udpTsReaderClose    (BITSTREAMREADER_HANDLE h);

/// \brief Request a data block from a bitstream reader.
/// \param h Handle to a bitstraem reader
/// \return buf Pointer to the data buffer
/// \return len Pointer to the length of data in the buffer
/// \return au Pointer to the number of access units in the data block
/// \return type Pointer to the type of data in the data block
/// \return sid Pointer to the stream ID of the block
int                    udpTsReaderGetBlock (BITSTREAMREADER_HANDLE h, char **buf, int *len, int *au, 
                                          int *type, int *sid);

/// \brief Configure the bitstream reader
/// \param param Parameter index to configure
/// \param value Parameter value to set
/// \return 1 for Success, 0 for failure
int                    udpTsReaderConfig   (BITSTREAMREADER_HANDLE h, int param, long long value);

/// \brief This operation is not supported on the UDP reader
int                    udpTsReaderSwitchFile(BITSTREAMREADER_HANDLE h, int *srcId, int *codecId, int nStreams, int diffFile);

int                    udpTsReaderShow     (BITSTREAMREADER_HANDLE h);

/// \brief Set callback function pointer for the reader
/// \param h Handle to an open reader
/// \param fn Function pointer
/// \param arg Arguments for the callback function
int                    udpTsReaderSetCallback(BITSTREAMREADER_HANDLE h, BITSTREAMREADER_CALLBACK fn, void *arg);

/// \brief Ioctl for UDP reader
int                    udpTsReaderIOCTL    (BITSTREAMREADER_HANDLE h, int cmd, void *arg);

#endif /* __UDPTSREADER_H */
