/*******************************************************************************
*
* The content of this file or document is CONFIDENTIAL and PROPRIETARY
* to Mobilygen Corporation.  It is subject to the terms of a
* License Agreement between Licensee and Mobilygen Corporation.
* restricting among other things, the use, reproduction, distribution
* and transfer.  Each of the embodiments, including this information and
* any derivative work shall retain this copyright notice.
*
* Copyright 2003 Mobilygen Corporation
* All rights reserved.
*
* QuArc is a registered trademark of Mobilygen Corporation.
*
*******************************************************************************/

/*******************************************************************************
* @(#) $Id$
*******************************************************************************/

/// \defgroup wsaa_wsgroup Writestreamer
/// \{

////////////////////////////////////////////////////////////////////////////////
///
/// \page writestreamer WriteStreamer
/// \latexonly
/// \chapter{WriteStreamer}
/// \endlatexonly
///
/// The writestreamer is a host-only object that is responsible for reading
/// bitstreams from the codec, formatting them appropriately, and then storing
/// or transmitting them to the ultimate destination.  The writestreamer
/// manages a set of constructs called "bitstream writers", each of which
/// is capable of formatting a bitstream into a specific type.  For example,
/// there is a MP4 bitstream writer, an RTP bitstream writer, and a elementary
/// AVC bitstream writer, among others.  The writestreamer can handle multiple
/// streams from the codec concurrently and can assign one or more bitstream
/// writers to each stream.  For instance, the codec could be generating two
/// streams A and B and the writestreamer could be storing stream A as an MP4,
/// be sending A as an RTP stream and storing B as an elementary AVC stream.
///
/// Other than storing/transmitting just the bitstream block received from the
/// codec, it can also send some metadata to each writer. The metadata can
/// be constructed from events received from the codec (like those for object
/// tracking) and passed on to write streamer with the function
/// WriteStreamer_WriteMetadata().
///
/// The writestreamer is instantiated with the WriteStreamer_Create() C call
/// or via the sysctl0:createWriteStreamer() LUA function.  If metadata is
/// enabled, then the queuesize passed must be increased as the same queue
//  will now hold bitstream blocks as well as metadata.
/// The writestreamer object embeds a single thread which must be started with
/// the WriteStreamer_Start() C function or the :start() LUA function.  If
/// the writestreamer is created as a LUA object ws0 then you would start
/// it with ws0:start().
///
/// Once started, bitstream writers can be opened and closed.  There are
/// multiple types of bitstream writers, each type of which is capable of
/// creating a specific media type.  The writestreamer publishes the types
/// as strings such as "mp4", "rtp".  Bitstream writers are opened using
/// the C function WriteStreamer_OpenBitstreamWriterByType or using the
/// LUA function :open.  The function WriteStreamer_OpenBitstreamWriterByType takes
/// as its arguments the type (string), the filename and a list of stream IDs.
/// When the writestreamer receives stream data from the codec, it will send
/// the data to all open bitstream writers which have expressed interest in that
/// stream id.  Note that a bitstream writer can express interest in multiple or
/// all streams, and multiple bitstream writers can express interest in the same
/// stream.
///
/// Similarly, you can close an open bitstream writer using the C function
/// WriteStreamer_CloseBitstreamWriterByType or the LUA function :close.
/// The function takes as its argument the type and filename which uniquely
/// identifies the writer to the writestreamer.
///
/// Bitstream writers can be opened or closed at any time including before
/// the encoder itself has been started.  All bitstream writers should be
/// written in such a way that they will self-synchronize to the stream
/// and create a valid output.  For example, a bitstream writer that
/// handles AVC streams would wait for an SPS/PPS before handling slice
/// data.
///
/// As mentioned earlier each bitstream writer can be opened with the
/// WriteStreamer_OpenBitstreamWriterByType function which takes the
/// filename to be opened.  For simple file storage, the filename is self-
/// evident but there are other bitstream writers where more information
/// is needed other than filename.  The mechanism used to communicate these
/// options is via a comma-separated list.  For example, the MP4 writer
/// has an option to generate multiple output files of a specified duration.
/// The user specifies the base filename with "fn=<name>" and the length
/// with "len=<length>" to create a string (for example)
/// "fn=/path/to/myfile.mp4,len=1800" which would create files of 60 seconds
/// (1800 frames) each.
///
/// \section zerocopy Zero-copy Operation
///
/// Significant performance gains can be obtained when mobiapp is run on
/// the MG3500's internal ARM due to the unified memory architecture.
/// Specifically, the writestreamer does not have to copy bitstream data
/// from codec memory to a local buffer and can directly refer to the
/// bitstream data in the codec's memory.  This optimization makes use
/// of the Linux mmap() call to memory map in the memory used by the codec.
///
/// Furthermore, bitstream writers can be enhanced to support zero copy
/// operation through the use of sendfile().  (At this time, only the
/// netwriter and rtpwriter writers have been enhanced with this feature).
///
/// For copy operation, each bitstream writer must supply a NewBlock
/// function pointer.  This function is called each time there is a new
/// bitstream block available.  Bitstream writers can optionall provide
/// a NewBlockUMA function pointer which is invoked when zero copy
/// operation has been requested.
///
/// The constructor of the writestreamer takes two flags as arguments.
/// The first flag uma tells the writestreamer to directly use codec
/// memory instead of copying the data.  The second flag sendfile
/// tells the the writestreamer to call the NewBlockUMA function.
/// Both of these flags are defaulted to 1 for the MG3500 ARM and
/// to 0 for all other processors.  Note that it is valid to have
/// uma=1 and sendfile=0 as this will result in bitstream writers
/// not using sendfile() but still using direct pointers.
///
/// Typical operation therefore is for SOC systems to use uma=1 and
/// sendfile=1, coprocessor systems would typically use uma=0 and
/// sendfile=0.  Note that the uma flag and sendfile flags are available
/// in the LUA bindings as part of the constructor.
///
/// ws = sysctl0:createWriteStreamer({name="ws0", uma=1, sendfile=1});
///
/// Furthermore, the writestreamer's open function takes a set of flags
/// as an argument.  One flag, Q_WS_CFP_FLAG_DISABLE_UMA when set will
/// forcibly disable the call of NewBlockUMA on a per writer basis
/// irrespective of the uma and sendfile settings.
///
/// \section wsstatus Status block and query information
///
/// The writestreamer's query information is not derived from the codec but
/// is instead internally generated.  The description of the status fields
/// are described below.
///
/// \latexonly
/// \input{structWRITESTREAMER__QUERY__DATA}
/// \endlatexonly

#ifndef __WRITESTREAMER_H
#define __WRITESTREAMER_H

#include <qhal_em.h>

#include "bswriter.h"

/// \brief Maximum path length that can be given to the write streamer
#define MAX_FILENAME         (4096)

/// \brief Number of outstanding bitstream blocks in the pending queue - not used anymore as it is
/// now runtime configurable with WriteStreamer_Create()
#define WRSTREAMER_QUEUESIZE (100)

/// \brief Maximum Metadata size in bytes - may make it configurable later on
/// It will contain maximum length of a single metadata entry
/// Used for wrapping in the separate metadata queue
#define WRSTREAMER_METADATA_MAX_SIZE (512)

/// \brief Metadata queue size in bytes - may make it configurable later on
/// It will contain variable length metadata entries
/// The worst case size can be object tracking rectangle info for 30 objects
/// Each object has 4 integers each for x, y, width and height.
/// So maximum = 30 * 4 * 4 = 480
/// Assuming average is 10 objects per frame then average size = 160 bytes
#define WRSTREAMER_METADATA_QUEUESIZE (16000)

/// \brief Size of internal buffer used to reading data from the codec
#define WRSTREAMER_BLOCKSIZE (65536)

/// \brief Maximum number of bitstream writers allowed
#define MAX_BITSTREAMWRITERS (32)

/// \brief Maximum number of bitstream block reads under bitstream aggregation mode
#define MAX_MULTIBLOCK_READ  (32)

/// \brief Handle to a writestreamer object that is returned by the
/// Create function.
typedef void *WRITESTREAMER_HANDLE;

typedef struct
{
    int id;
    /// \brief Verbose description of internal thread state
    char state[256];
    /// \brief Short term aggregate bitrate
    int shortTermBitrate;
    /// \brief Bitrate since start() was called
    int longTermBitrate;
    /// \brief Current number of entries in the bitstream block queue
    int bblockQueueFullness;
    /// \brief Maximum size of the bitstream block queue
    int queueSize;
    /// \brief Number of blocks/second written to the queue in the last 2 seconds
    int writeBlockRate;
    /// \brief Number of DMAs per second
    int dmaRate;
    /// \brief Total load average of the writestreamer thread
    int usage;
    /// \brief Thread Load average due to reading bitstreams from the codec
    int codecusage;
    /// \brief Thread load average due to all bitstream writers
    int bsusage;
    /// \brief Thread load average due to acknowledging codec blocks
    int ackusage;
    /// \brief Verbose list of supported bitstream writer types
    char supportedTypes[256];
    /// \brief Verbose list of all open bitstream writers
    char openWriters[MAX_BITSTREAMWRITERS][1024];
    /// \brief Internal buffer size
    int bufferSize;


} WRITESTREAMER_QUERY_DATA;


struct _wsbblock;

/// \brief Type for callback issued when a bitstream block has completed sending a block
/// to all bitstream writers who have expressed interest in the block.
typedef void (*WRITESTREAMER_BLOCKDONE_CALLBACK)(struct _wsbblock *, void *arg);

/// \brief Enable or disable bitrate on commandline
#define Q_WS_CFG_SHOW_BITRATE               1

/// \brief Enable the fetching of bitstreams and sending to bitstreams writers.  If
/// disabled it will handle the events only.
#define Q_WS_CFG_READ_BITSTREAMS            2

/// \brief Enable reading multiple continguous bitstream blocks in one memory read
#define Q_WS_CFG_MULTISTREAM_GATHER_SUPPORT 3

/// \brief Adjust the size of internal buffer used to reading data from the codec
#define Q_WS_CFG_READ_BLOCKSIZE             4

/// \brief Enable writing metadata to the work queue for sending to bitstreams writers.
/// Disabled by default.
#define Q_WS_CFG_METADATA                   5

/// \brief Set the SCHED_FIFO priority for this thread. Needs to be called before starting.
/// Set to 0 to disable. Disabled by default. POTENTIALLY DANGEROUS, KNOW WHAT YOU ARE DOING.
#define Q_WS_CFG_THREAD_PRIORITY            6

/// \brief Enable or disable single write bypass mode.
#define Q_WS_CFG_SINGLE_WRITER_BYPASS_MODE  7

#define Q_WS_CFG_MAX_CFG                    8

/// \brief Forcibly disables UMA on a bitstream writer even if ws is configured
/// to use it.
#define Q_WS_CFP_FLAG_DISABLE_UMA   1

/// \brief Enables sending of metadata to a bitstream writer.
/// Note that that writing metadata to work queue must be enabled using Q_WS_CFG_METADATA for this to work.
#define Q_WS_CFP_FLAG_ENABLE_METADATA   2


/// \brief Ioctl command to save pre and post buffer in circular bitstream writer to files pointed by arg
#define Q_WS_IOCTL_CIRCULARWRITER_SAVE      1

/// \brief Ioctl command to save a JPEG snapshot to the specified file in the next NewBlock call of JPEG writer
#define Q_WS_IOCTL_JPEGWRITER_SAVE      2

/// \brief Ioctl command to start publishing rtmp stream
#define Q_WS_IOCTL_RTMPWRITER_CONNECT      3

/// \brief Ioctl command to teardown rtmp stream
#define Q_WS_IOCTL_RTMPWRITER_DISCONNECT      4

/// \brief Ioctl command to get rtmp publish state
#define Q_WS_IOCTL_RTMPWRITER_GETSTATE      5

/// \brief Ioctl command to set/get rtmp writer settings
#define Q_WS_IOCTL_RTMPWRITER_SETTINGS      6

/// \brief Ioctl command to change host/port for TDSWriter plugin
#define Q_WS_CHANGE_HOST                       31

/// \brief Ioctl command to query client state for TDSWriter plugin (0-not connected; 1-connected and receiving stream)
#define Q_WS_GET_CLIENT_STATE                  32

/// \brief A set of commands used by upper level to control the bitstream writers
typedef enum
{
    BSW_CMD_DUMP_METADATA,
    NUM_BSW_CMD
} BSW_CMD;

/// \brief Enumerations identifying the type of data in queue - bitstream block or metadata
typedef enum
{
    WS_BBLOCK,
    WS_METADATA
} wsDataType_t;

/// \brief Existing metadata types
typedef enum
{
    //Start with 1
    META_OBJTRACKING=1,
    META_MOTIONALARMS,
    META_DIGITALSIGNATURE,
    META_SRTSUBTITLES
} metadataType_t;

/// \brief Bitstream block structure that is used to pass work requests from the event handler
/// (which is part of the writestreamer but run in the event handler thread
/// context) to the actual writestreamer's threadproc.
typedef struct _wsbblock
{
    /// \brief Indicates which codec this bitstream block came from
    int codec;

    /// \brief The base address in codec memory of the bitstream block
    int baseAddress;

    /// \brief The size of the block
    int blockSize;

    /// \brief Stream ID of this block
    int streamID;

    /// \brief A flag that indicates if this is the last block in a file.  This
    /// flag is set by the codec after a flush operation.
    int lastBlock;

    /// \brief callback for block done
    WRITESTREAMER_BLOCKDONE_CALLBACK blockDoneCallback;

    /// \brief Type for callback issued when a bitstream block has completed sending a block
    /// to all bitstream writers who have expressed interest in the block.
    void *blockDoneCallbackArg;

    /// \brief Timestamp when block was about to be written to writestreamer
    unsigned int timestamp;

} WRITESTREAMER_BBLOCK;

/// \brief Metadata structure that is used to pass work requests from the event handler
/// (which is part of the writestreamer but run in the event handler thread
/// context) to the actual writestreamer's threadproc.
typedef struct _wsmblock
{
    /// \brief Size of Metadata pointed by dataPtr
    int dataSize;

    /// \brief Type of Metadata
    metadataType_t dataType;

    /// \brief Stream ID
    int streamID;

    /// \brief Metadata contents - points to the actual data
    void *dataPtr;

} WRITESTREAMER_METADATA;

/// \brief Structure that is used to pass work requests from the event handler
/// (which is part of the writestreamer but run in the event handler thread
/// context) to the actual writestreamer's threadproc.
typedef struct _wsquedata
{
    /// \brief Data type - either bitstream block or metadata
    wsDataType_t dataType;

    union {
       /// \brief bitstream block
       WRITESTREAMER_BBLOCK   bblock;
       /// \brief Metadata block
       WRITESTREAMER_METADATA metadata;
    };

} WRITESTREAMER_QUEUEDATA;

typedef struct
{
    int                         nBlocks;
    WRITESTREAMER_QUEUEDATA     bblockList[MAX_MULTIBLOCK_READ];
} WRITESTREAMER_BBLOCK_LIST;

/// \brief Queue which contains either pending bitstream block write requests
/// or metadata pointers. The actual metadata is in a separate variable sized list
/// which ia accessed using the indices metadataRdPtr and metadataWrPtr.
/// This queue connects the event handler function running in the event handler thread
/// to the threadproc of the writestreamer.
typedef struct
{
    /// \brief Internal queue storage
    WRITESTREAMER_BBLOCK_LIST      *queue;

    /// \brief Metadata read pointer owned by the writestreamer thread
    int                             metadataRdPtr;

    /// \brief Metadata write pointer owned by the WriteBBlock method
    int                             metadataWrPtr;

    /// \brief Queue read pointer owned by the writestreamer thread
    int                             rdPtr;

    /// \brief Write pointer owned by the WriteBBlock method
    int                             wrPtr;

    /// \brief Updown counter containing queue fullness.  Used as predicate
    /// (non-zero) to indicate that there is work for the main thread.
    int                             fullness;

    /// \brief Mutex which protects condvar
    pthread_mutex_t                 mutex;

    /// \brief Condition variable which is signalled to indicate that there
    /// has been a new bitstream block written to the queue
    pthread_cond_t                  condvar;

} BBLOCK_METADATA_QUEUE;

/// \brief Max number of streams a single writer can accept
#define MAX_STREAMS_PER_WRITER      (16)

/// \brief C Constructor for the writestreamer
/// \param codec The index of the codec from which streams will be read.  Note
/// that this is used only in multi-codec systems.
/// \param bufferSize In the case of non-UMA operation, this is the size of the
/// local buffer
/// \param uma 1 to enable unified memory access, 0 to disable
/// \param sendfile 1 to enable sendfile use in bitstream writers (along with uma = 1), 0 to disable
/// \param queueSize Size of internal queue between calling thread and internal thread
/// \return A handle to the newly created object.
WRITESTREAMER_HANDLE WriteStreamer_Create(int codec, int bufferSize, int uma, int sendfile, int queueSize);

/// \brief C Destructor for the writestreamer (not supported)
void    WriteStreamer_Terminate( void );

/// \brief Starts execution of the writestreamer's embedded thread
/// \param h Handle to a writestreamer object
/// \return 0 for failure, 1 for success
int     WriteStreamer_Start( WRITESTREAMER_HANDLE h );

/// \brief Function that write a WRITESTREAMER_BBLOCK struct to the embedded queue
/// that terminates a the threadproc.
/// \param h Handle to a writestreamer object
/// \param bblock Pointer to a WRITESTREAMER_BBLOCK structure for the writestreamer thread to handle
int     WriteStreamer_WriteBBlock( WRITESTREAMER_HANDLE h, WRITESTREAMER_BBLOCK *bblock );

/// \brief Function that write a WRITESTREAMER_METADATA struct to the embedded queue
/// that terminates a the threadproc.
/// \param h Handle to a writestreamer object
/// \param bblock Pointer to a WRITESTREAMER_METADATA structure for the writestreamer thread to handle
int     WriteStreamer_WriteMetadata( WRITESTREAMER_HANDLE h, WRITESTREAMER_METADATA *metadata );

/// \brief A debug function that disables the demux of streams and passes all streams to all
/// bitstream writers.
/// \param h Handle to a writestreamer object
/// \param enable 0 to disable, 1 to enable
/// \return 0 for failure, 1 for success
int     WriteStreamer_DemuxStreams( WRITESTREAMER_HANDLE h, int enable );

/// \brief Configure one entry of the set of configuration parameters
/// \param h Handle to a writestreamer object
/// \param Index Index of the configuration parameter in the table
/// \param Value Value of the configuration parameter to set
/// \return 0 for failure, 1 for success
int     WriteStreamer_Configure( WRITESTREAMER_HANDLE h, int index, int value);

/// \brief This function opens a bitstream writer of a specified type.  The caller specifies a list of stream ID's
/// that the writer will be assigned (which are not mututally exclusive with other writers), the bitstream
/// writer type (which is a unique string), and a filename.
/// \param h Handle to a writestreamer object
/// \param type Pointer to a string that specifies the type of writer to open.  For example "file" opens the file writer.
/// \param numStreams The number of streams specified by the streamIds pointer
/// \param streamIds Pointer to an array of stream IDs of size numStreams
/// \param filename Pointer to a string specifying either the destination filename or comma-separated arguments
/// \return idx Index of the opened bitstream writer
///
/// The WriteStreamer_OpenBitstreamWriterByType function opens a bitstream writer type as specified by the
/// argument type.  The write streamer object maintains a translation table between type as expressed as as
/// string and a true object.  The external API uses string to specify type which is subsequently translated to
/// a true object.
///
/// Filename is bitstream writer type-specific.  For example, most storage writers (meaning those that open
/// files and write a stream to the file) use the string as the full path to the destination file.  However,
/// many bitstream writers have optional or mandatory arguments that must be specified.  The convention is
/// to specify arguments as a comma-separated list.  For example, a writer may take the argument "fn" which
/// specifies filename and "bs" which specifies buffer size. The writer would then be opened with the
/// filename "fn=/path/to/file,bs=1024".
int WriteStreamer_OpenBitstreamWriterByType(WRITESTREAMER_HANDLE h, char *type, int *streamIds, int numStreams, char *filename, unsigned int flags);

/// \brief This function closes an already opened bitstream writer.  The caller must specify both the type (as a
/// string) and the filename which was used to open the stream.  The result of the close operation is
/// bitstream writer specific.
/// \param h Handle to a writestreamer object
/// \param type Type of bitstream writer to close (as expressed as a string)
/// \param filename Filename that was used to open the bitstream writer
/// \return 0 for failure, 1 for success
int WriteStreamer_CloseBitstreamWriterByType( WRITESTREAMER_HANDLE h, char *type, char *filename );

/// \brief This function closes an already opened bitstream writer.  The caller must specify the bitstream writer
/// index.  The result of the close operation is bitstream writer specific.
/// \param h Handle to a writestreamer object
/// \param idx Bitstream writer index
/// \return 0 for failure, 1 for success
int WriteStreamer_CloseBitstreamWriterByIdx(WRITESTREAMER_HANDLE h, int idx);

/// \brief Displays verbose status information
/// \param h Handle to a writestreamer object
/// \return 0 for failure, 1 for success
int WriteStreamer_Query( WRITESTREAMER_HANDLE h, WRITESTREAMER_QUERY_DATA *data, int silent);

/// \brief This function adds a bitstream writer to the list of available writers
/// \param name Name of bitstream writer type
/// \param bsw Handle to bitstream writer
/// \return 0 for failure, 1 for success
/// \ingroup wsgroup
int WriteStreamer_AddBitstreamWriterTypeByName(char *name, BITSTREAMWRITER *bsw);

/// \brief This function switches writer from one file to another
/// \param h Handle to a writestreamer object
/// \param idx Index of an opened bitstream writer
/// \param newFile Name of new file that is going to be created
/// \return 0 for failure, 1 for success
int WriteStreamer_SwitchFile(WRITESTREAMER_HANDLE h, int idx, char *newFile);

/// \brief This function configures an already opened bitstream writer.  The effect of
/// each parameter is writer-specific.  Parameters available for configuration are
/// Q_WS_CFG_BSW_*.
/// \param h Handle to a writestreamer object
/// \param idx Index of an opened bitstream writer
/// \param param Parameter name to configure
/// \param value Value to which the parameter is set
/// \return 0 for failure, 1 for success
int WriteStreamer_ConfigureWriter(WRITESTREAMER_HANDLE h, int idx, int param, int value);

/// \brief This function sets the callback function for a bitstream writer specified by idx
/// \param h Handle to a writestreamer object
/// \param idx Index of an opened bitstream writer
/// \param fn Callback function
/// \param arg Function argument
/// \return 0 for failure, 1 for success
int WriteStreamer_SetWriterCallback(WRITESTREAMER_HANDLE h, int idx, BITSTREAMWRITER_CALLBACK fn, void *arg);

/// \brief This function sets the stream IDs for a bitstream writer specified by idx
/// \param h Handle to a writestreamer object
/// \param idx Index of an opened bitstream writer
/// \param streamIds Array of stream IDs that will be written out
/// \param numStreams Number of streams in the array
/// \return 0 for failure, 1 for success
int WriteStreamer_SetStreamID(WRITESTREAMER_HANDLE h, int idx, int *streamIds, int numStreams);

/// \}

/// \brief This function checks if metadata is enabled
/// \param h Handle to a writestreamer object
/// \return 0 if not enabled, 1 if enabled
int WriteStreamer_IsMetadataEnabled( WRITESTREAMER_HANDLE h );


/// \brief This function configures an already opened bitstream writer.  The effect of
/// each parameter is writer-specific.  Parameters available for configuration are
/// Q_WS_IOCTL_*.
/// \param h Handle to a writestreamer object
/// \param idx Index of an opened bitstream writer
/// \param cmd Ioctl command
/// \param arg String value to pass
/// \return 0 for failure, 1 for success
int WriteStreamer_Ioctl (WRITESTREAMER_HANDLE h, int idx, int cmd, void *arg);

/// \brief Displays verbose status information for each individual writer
/// \param h Handle to a writestreamer object
/// \return 0 for failure, 1 for success
int WriteStreamer_Show( WRITESTREAMER_HANDLE h );

/// \brief Reset minimum and maximum statistics
/// \param h Handle to a writestreamer object
/// \return 1 for success, 0 for failure
int WriteStreamer_ResetMinMax( WRITESTREAMER_HANDLE h );

#endif
