var
    _        = require('underscore'),
    exec     = require('child_process').exec,
    format   = require('util').format,
    ns       = require('../lib/ns'),
    fs       = require('fs'),
    deferred = require('deferred')
    //uploader = require('../lib/uploader')
    ;

module.exports = {

    /* --- Devices & CDNs --- */

    /**
     Will unload any currently loaded plugins for this session
     @method unloadAllPlugins
     @param {String} sid Serial number of the device
     */

    unloadAllPlugins: function (sid) {
        this.sess(sid).iface('E').enqueue('dev:unlink-all-plugins', function (session) {
            session.unloadAllPlugins()(function () {
                console.log('unloadAllPlugins() success');
                session = null;
            },function (err) {
                console.log('unloadAllPlugins() failed:', err.toString());
                session = null;
            }).done();
        });
    },

    /**
     Will clears out the plugins in the pluginsMap associated with the session
     only difference between this and "unloadAllPlugins" is that this does not
     execute the ioctl command to try and remove plugins
     @method unloadAllPlugins
     @param {String} sid Serial number of the device
     */

    unloadPluginsMap: function (sid) {
        if( this.sess(sid).iface('E') ){
            this.sess(sid).iface('E').enqueueOne('dev:unlink-all-plugins', function (session) {
                session.unloadPluginsMap();
            });
        }
    },

    /**
     Loads 'snap' plugin on Sputnik for a given device
     @method loadSnapshotPlugin
     @param {String} sid Serial number of the device
     */

    loadSnapshotPlugin: function (sid) {
        var def = deferred();
        var self = this;
        this.sess(sid).iface('E').enqueueOne('dev:load-snap-plugin', function (session) {
            self.plugins.loadSnap(session)(function (pluginId) {
                console.log('loadSnapshotPlugin() loaded snap plugin', pluginId);
                def.resolve(pluginId);
            },function (err) {
                console.log('loadSnapshotPlugin() could not load snap plugin', err.toString());
                def.resolve(new Error(err.toString()));
            }).done();
            session = null;
        });
        return def.promise;
    },

    /**
     Loads 'streamMonitor' plugin on Sputnik for a given device
     @method loadSnapshotPlugin
     @param {String} sid Serial number of the device
     */

    loadRTSPServerPlugin: function (sid, config) {
        var self = this;
        this.sess(sid).iface('E').enqueueOne('dev:load-RTSP-plugin', function (session) {
            self.plugins.loadRTSPServer(session, sid, config)(function (pluginId) {
                console.log('loadRTSPPlugin() loaded RTSP plugin', pluginId);
            },function (err) {
                console.log('loadRTSPPlugin() could not load RTSP plugin', err.toString());
            }).done();
            session = null;
        });
    },

    /**
     If decoder connects to this Sputnik for the first time, we assign it an unused UDPTS port
     @method updateUdptsPort
     @param {String} sid Serial number for device
     */

    updateUdptsPort: function (sid) {
        var self = this;

        this.sess(sid).iface('C').enqueueOne('dev:update-udpts-port', function (session) {
            session.getDeviceType()(function (type) {
                if (type === 'decoder') {
                    self.store.get('decoder-ports', function (err, data) {
                        if (err) return console.error('Store get error', err.stack || err.toString())

                        if (data[sid] === undefined) {
                            var ports = _.values(data),
                                port = (ports.length > 0) ? _.max(ports) + 1 : 5000,
                                config = ns({}, 'VideoPlayer.Settings.1.TS.Server.port', port.toString());

                            console.log('updateUdptsPort() new ports data................', sid, port.toString(), config);

                            data[sid] = port.toString();
                            self.store.save('decoder-ports', function (err) {
                                if (err) console.error('Store save error', err.stack || err.toString());
                            });

                            session.setSettings(config)(function (data) {
                                console.log('updateUdptsPort() updated decoder port');
                                session = null;
                            }, function (err) {
                                console.log('updateUdptsPort() failed to update decoder port', err.toString());
                            });
                        } else {
                            // make sure to update to the current port, just in case it's different
                        }
                    });
                }
            },function (err) {
                console.log('updateUdptsPort() failed to get type');
            }).done();
        });
    },

    /**
     Logs memory usage
     @method memWatch
     */

    memWatch: function () {
        var self = this,
            pid = process.pid,
            mem = process.memoryUsage().rss,
            uptime = process.uptime(),
            memMb = Math.round(((mem / 1024) / 1024));

        this._mem = this._mem || memMb;

        exec('ls /proc/' + pid + '/fd/ | wc -l', function (err, stdout, stderr) {
            stdout = stdout.replace(/(\r\n|\n|\r)/gm, '');

            var uptimeM = Math.floor(uptime / 60),
                uptimeS = Math.floor(uptime - uptimeM * 60),
                str = format('  RAM: %s Mb (5 sec Δ %s Mb); Uptime: %s:%s; Open files: %s', memMb, memMb - self._mem, uptimeM, uptimeS, stdout);

            self._mem = memMb;
            console.log(str);
        });
    },

    /*uploadRecordsToCore: function () {
        var self = this;

        this.store.get('records', function (err, records) {
            if (err) return console.error('<uploadRecordsToCore> Error.', err.stack || err.toString());
            _.each(records, function (record, recordId) {
                if (record.upload && record.status === 'recorded') uploader.upload(self, record);
            });


        });

    }*/
};

